@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -123,34 +123,41 @@ environment variables:
 
 '''
 
-from __future__ import (print_function, unicode_literals)
+from __future__ import print_function, unicode_literals, division, absolute_import
 
-import os, sys, json, time, logging, platform, collections, ssl, traceback
+import os, sys, json, time, logging, platform, ssl, traceback
 import errno
 import requests
 import socket
-
-from requests.exceptions import ConnectionError, HTTPError, Timeout
+from collections import namedtuple
+from . import exceptions
 from requests.auth import AuthBase
+from requests.packages import urllib3
+from requests.packages.urllib3.packages.ssl_match_hostname import match_hostname
 from .compat import USING_PYTHON2, expanduser
 
 logger = logging.getLogger(__name__)
 logger.addHandler(logging.NullHandler())
-logging.getLogger('dxpy.packages.requests.packages.urllib3.connectionpool').setLevel(logging.ERROR)
 
-from . import exceptions
+
+def configure_urllib3():
+    # Disable verbose urllib3 warnings and log messages
+    urllib3.disable_warnings(category=urllib3.exceptions.InsecurePlatformWarning)
+    logging.getLogger('dxpy.packages.requests.packages.urllib3.connectionpool').setLevel(logging.ERROR)
+
+    # Trust DNAnexus S3 upload tunnel
+    def _match_hostname(cert, hostname):
+        if hostname == "ul.cn.dnanexus.com":
+            hostname = "s3.amazonaws.com"
+        match_hostname(cert, hostname)
+
+    urllib3.connection.match_hostname = _match_hostname
+
+configure_urllib3()
+
 from .toolkit_version import version as TOOLKIT_VERSION
 __version__ = TOOLKIT_VERSION
 
-snappy_available = True
-if sys.version_info < (3, 0):
-    try:
-        import snappy
-    except ImportError:
-        snappy_available = False
-else:
-    snappy_available = False
-
 API_VERSION = '1.0.0'
 AUTH_HELPER, SECURITY_CONTEXT = None, None
 JOB_ID, WORKSPACE_ID, PROJECT_CONTEXT_ID = None, None, None
@@ -163,18 +170,43 @@ APISERVER_PROTOCOL = DEFAULT_APISERVER_PROTOCOL
 APISERVER_HOST = DEFAULT_APISERVER_HOST
 APISERVER_PORT = DEFAULT_APISERVER_PORT
 
-SESSION_HANDLERS = collections.defaultdict(requests.session)
-
 DEFAULT_RETRIES = 6
+DEFAULT_TIMEOUT = 600
+DEFAULT_RETRY_AFTER_503_INTERVAL = 60
+
 _DEBUG = 0  # debug verbosity level
 _UPGRADE_NOTIFY = True
 
 USER_AGENT = "{name}/{version} ({platform})".format(name=__name__,
                                                     version=TOOLKIT_VERSION,
                                                     platform=platform.platform())
-
+_default_headers = requests.utils.default_headers()
+_default_headers['DNAnexus-API'] = API_VERSION
+_default_headers['User-Agent'] = USER_AGENT
+_default_timeout = urllib3.util.timeout.Timeout(connect=DEFAULT_TIMEOUT, read=DEFAULT_TIMEOUT)
+_pool_manager = urllib3.PoolManager(maxsize=32,
+                                    cert_reqs=ssl.CERT_REQUIRED,
+                                    ca_certs=requests.certs.where(),
+                                    headers=_default_headers,
+                                    timeout=_default_timeout)
+_RequestForAuth = namedtuple('_RequestForAuth', 'method url headers')
 _expected_exceptions = exceptions.network_exceptions + (exceptions.DXAPIError, )
 
+def _get_pool_manager(request_kwargs):
+    if 'verify' in request_kwargs or 'DX_CA_CERT' in os.environ:
+        cert_reqs = ssl.CERT_REQUIRED
+        ca_certs = request_kwargs.get('verify', os.environ.get('DX_CA_CERT'))
+        if request_kwargs.get('verify') is False or os.environ.get('DX_CA_CERT') == 'NOVERIFY':
+            cert_reqs, ca_certs = ssl.CERT_NONE, None
+            urllib3.disable_warnings()
+        return urllib3.PoolManager(cert_reqs=cert_reqs,
+                                   ca_certs=ca_certs,
+                                   headers=_default_headers,
+                                   timeout=_default_timeout)
+    else:
+        return _pool_manager
+
+
 def _process_method_url_headers(method, url, headers):
     if callable(url):
         _url, _headers = url()
@@ -213,21 +245,43 @@ def _is_retryable_exception(e):
     have been established, we return False.
 
     """
+    if isinstance(e, urllib3.exceptions.ProtocolError):
+        e = e.args[1]
+    if isinstance(e, (socket.gaierror, socket.herror)):
+        return True
+    if isinstance(e, socket.error) and e.errno in _RETRYABLE_SOCKET_ERRORS:
+        return True
+    return False
+
+def _extract_msg_from_last_exception():
+    ''' Extract a useful error message from the last thrown exception '''
+    last_exc_type, last_error, last_traceback = sys.exc_info()
+    if isinstance(last_error, exceptions.DXAPIError):
+        # Using the same code path as below would not
+        # produce a useful message when the error contains a
+        # 'details' hash (which would have a last line of
+        # '}')
+        return last_error.error_message()
+    else:
+        return traceback.format_exc().splitlines()[-1].strip()
+
+
+def _extract_retry_after_timeout(response):
+    '''Returns the time in seconds that the server is asking us to
+    wait. The information is deduced from the server http response.'''
     try:
-        if isinstance(e, ConnectionError):
-            # Unfortunately requests doesn't seem to provide a sensible
-            # API to retrieve the cause
-            cause = e.args[0].args[1]
-            if isinstance(cause, (socket.gaierror, socket.herror)):
-                return True
-            if isinstance(cause, socket.error) and cause.errno in _RETRYABLE_SOCKET_ERRORS:
-                return True
-        return False
-    except (AttributeError, TypeError, IndexError):
-        return False
+        seconds_to_wait = int(response.headers.get('retry-after', DEFAULT_RETRY_AFTER_503_INTERVAL))
+    except ValueError:
+        # retry-after could be formatted as absolute time
+        # instead of seconds to wait. We don't know how to
+        # parse that, but the apiserver doesn't generate
+        # such responses anyway.
+        seconds_to_wait = DEFAULT_RETRY_AFTER_503_INTERVAL
+    return max(1, seconds_to_wait)
 
 
-def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True, timeout=None,
+def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
+                  timeout=DEFAULT_TIMEOUT,
                   use_compression=None, jsonify_data=True, want_full_response=False,
                   decode_response_body=True, prepend_srv=True, session_handler=None,
                   max_retries=DEFAULT_RETRIES, always_retry=False, **kwargs):
@@ -238,13 +292,17 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True, timeou
     :type data: list or dict, if *jsonify_data* is True; or string or file-like object, otherwise
     :param headers: Names and values of HTTP headers to submit with the request (in addition to those needed for authentication, compression, or other options specified with the call).
     :type headers: dict
-    :param auth: Overrides the *auth* value to pass through to :meth:`requests.request`. By default a token is obtained from the ``DX_SECURITY_CONTEXT``.
+    :param auth:
+        Controls the ``Authentication`` header or other means of authentication supplied with the request. If ``True``
+        (default), a token is obtained from the ``DX_SECURITY_CONTEXT``. If the value evaluates to false, no action is
+        taken to prepare authentication for the request. Otherwise, the value is assumed to be callable, and called with
+        three arguments (method, url, headers) and expected to prepare the authentication headers by reference.
     :type auth: tuple, object, True (default), or None
     :param timeout: HTTP request timeout, in seconds
     :type timeout: float
     :param config: *config* value to pass through to :meth:`requests.request`
     :type config: dict
-    :param use_compression: "snappy" to use Snappy compression, or None
+    :param use_compression: Deprecated
     :type use_compression: string or None
     :param jsonify_data: If True, *data* is converted from a Python list or dict to a JSON string
     :type jsonify_data: boolean
@@ -254,6 +312,7 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True, timeou
     :type decode_response_body: boolean
     :param prepend_srv: If True, prepends the API server location to the URL
     :type prepend_srv: boolean
+    :param session_handler: Deprecated.
     :param max_retries: Maximum number of retries to perform for a request. A "failed" request is retried if any of the following is true:
 
                         - A response is received from the server, and the content length received does not match the "Content-Length" header.
@@ -280,8 +339,6 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True, timeou
        through to :func:`DXHTTPRequest`.
 
     '''
-    if session_handler is None:
-        session_handler = SESSION_HANDLERS[os.getpid()]
     if headers is None:
         headers = {}
 
@@ -289,8 +346,18 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True, timeou
 
     url = APISERVER + resource if prepend_srv else resource
     method = method.upper() # Convert method name to uppercase, to ease string comparisons later
-    if _DEBUG >= 2:
-        print(method, url, "=>\n" + json.dumps(data, indent=2), file=sys.stderr)
+    if _DEBUG >= 3:
+        try:
+            formatted_data = json.dumps(data, indent=2)
+        except UnicodeDecodeError:
+            formatted_data = "<binary data>"
+        print(method, url, "=>\n" + formatted_data, file=sys.stderr)
+    elif _DEBUG == 2:
+        try:
+            formatted_data = json.dumps(data)
+        except UnicodeDecodeError:
+            formatted_data = "<binary data>"
+        print(method, url, "=>", formatted_data, file=sys.stderr)
     elif _DEBUG > 0:
         from repr import Repr
         print(method, url, "=>", Repr().repr(data), file=sys.stderr)
@@ -298,26 +365,14 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True, timeou
     if auth is True:
         auth = AUTH_HELPER
 
-    if 'verify' not in kwargs and 'DX_CA_CERT' in os.environ:
-        kwargs['verify'] = os.environ['DX_CA_CERT']
-        if os.environ['DX_CA_CERT'] == 'NOVERIFY':
-            kwargs['verify'] = False
-            from requests.packages import urllib3
-            urllib3.disable_warnings()
+    if auth:
+        auth(_RequestForAuth(method, url, headers))
 
     if jsonify_data:
         data = json.dumps(data)
         if 'Content-Type' not in headers and method == 'POST':
             headers['Content-Type'] = 'application/json'
 
-    headers['DNAnexus-API'] = API_VERSION
-    headers['User-Agent'] = USER_AGENT
-
-    if use_compression == 'snappy':
-        if not snappy_available:
-            raise exceptions.DXError("Snappy compression requested, but the snappy module is unavailable")
-        headers['accept-encoding'] = 'snappy'
-
     # If the input is a buffer, its data gets consumed by
     # requests.request (moving the read position). Record the initial
     # buffer position so that we can return to it if the request fails
@@ -326,17 +381,16 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True, timeou
     if hasattr(data, 'seek') and hasattr(data, 'tell'):
         rewind_input_buffer_offset = data.tell()
 
-    last_exc_type, last_error, last_traceback = None, None, None
-    time_started = time.time() if timeout else None
     try_index = 0
     while True:
-        success, streaming_response_truncated = True, False
+        success, streaming_response_truncated, time_started = True, False, None
         response = None
         try:
+            if _DEBUG > 0:
+                time_started = time.time()
             _method, _url, _headers = _process_method_url_headers(method, url, headers)
-            _timeout = timeout or 600
-            response = session_handler.request(_method, _url, headers=_headers, data=data, timeout=_timeout, auth=auth,
-                                               **kwargs)
+            response = _get_pool_manager(kwargs).request(_method, _url, headers=_headers, body=data,
+                                                         timeout=timeout, retries=False, **kwargs)
 
             if _UPGRADE_NOTIFY and response.headers.get('x-upgrade-info', '').startswith('A recommended update is available') and not os.environ.has_key('_ARGCOMPLETE'):
                 logger.info(response.headers['x-upgrade-info'])
@@ -349,43 +403,46 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True, timeou
 
             # If an HTTP code that is not in the 200 series is received and the content is JSON, parse it and throw the
             # appropriate error.  Otherwise, raise the usual exception.
-            if response.status_code // 100 != 2:
+            if response.status // 100 != 2:
                 # response.headers key lookup is case-insensitive
                 if response.headers.get('content-type', '').startswith('application/json'):
-                    content = json.loads(response.content.decode('utf-8'))
-                    error_class = getattr(exceptions, content["error"]["type"], exceptions.DXAPIError)
-                    raise error_class(content, response.status_code)
-                response.raise_for_status()
+                    content = json.loads(response.data.decode('utf-8'))
+                    try:
+                        error_class = getattr(exceptions, content["error"]["type"], exceptions.DXAPIError)
+                    except (KeyError, AttributeError, TypeError):
+                        error_class = exceptions.HTTPError
+                    raise error_class(content, response.status)
+                raise exceptions.HTTPError("{} {}".format(response.status, response.reason))
 
             if want_full_response:
                 return response
             else:
                 if 'content-length' in response.headers:
-                    if int(response.headers['content-length']) != len(response.content):
+                    if int(response.headers['content-length']) != len(response.data):
                         range_str = (' (%s)' % (headers['Range'],)) if 'Range' in headers else ''
                         raise exceptions.ContentLengthError(
                             "Received response with content-length header set to %s but content length is %d%s" %
-                            (response.headers['content-length'], len(response.content), range_str)
+                            (response.headers['content-length'], len(response.data), range_str)
                         )
 
-                if use_compression and response.headers.get('content-encoding', '') == 'snappy':
-                    # TODO: check if snappy raises any exceptions on truncated response content
-                    content = snappy.uncompress(response.content)
-                else:
-                    content = response.content
+                content = response.data
 
                 if decode_response_body:
                     content = content.decode('utf-8')
                     if response.headers.get('content-type', '').startswith('application/json'):
                         try:
                             content = json.loads(content)
-                            if _DEBUG >= 2:
-                                t = int(response.elapsed.total_seconds()*1000)
-                                print(method, url, "<=", response.status_code, "(%dms)"%t, "\n" + json.dumps(content, indent=2), file=sys.stderr)
+                            if _DEBUG > 0:
+                                t = int((time.time() - time_started) * 1000)
+                            if _DEBUG >= 3:
+                                print(method, url, "<=", response.status, "(%dms)" % t,
+                                      "\n" + json.dumps(content, indent=2), file=sys.stderr)
+                            elif _DEBUG == 2:
+                                print(method, url, "<=", response.status, "(%dms)" % t, json.dumps(content),
+                                      file=sys.stderr)
                             elif _DEBUG > 0:
-                                t = int(response.elapsed.total_seconds()*1000)
-                                print(method, url, "<=", response.status_code, "(%dms)"%t, Repr().repr(content), file=sys.stderr)
-                            return content
+                                print(method, url, "<=", response.status, "(%dms)" % t, Repr().repr(content),
+                                      file=sys.stderr)
                         except ValueError:
                             # If a streaming API call (no content-length
                             # set) encounters an error it may just halt the
@@ -394,32 +451,21 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True, timeou
                             # the client sees unparseable JSON, and we
                             # should be able to recover.
                             streaming_response_truncated = 'content-length' not in response.headers
-                            raise HTTPError("Invalid JSON received from server")
+                            raise exceptions.HTTPError("Invalid JSON received from server")
                 return content
             raise AssertionError('Should never reach this line: expected a result to have been returned by now')
         except Exception as e:
+            # Avoid reusing connections in the pool, since they may be
+            # in an inconsistent state (observed as "ResponseNotReady"
+            # errors).
+            _get_pool_manager(kwargs).clear()
             success = False
-            if timeout and time.time() - time_started > timeout:
-                logger.error("{} {}: Timeout exceeded".format(method, url))
-            elif isinstance(e, _expected_exceptions):
-                last_exc_type, last_error, last_traceback = sys.exc_info()
-                exception_msg = traceback.format_exc().splitlines()[-1].strip()
-
-                if response is not None and response.status_code == 503:
-                    DEFAULT_RETRY_AFTER_INTERVAL = 60
-                    try:
-                        seconds_to_wait = int(response.headers.get('retry-after', DEFAULT_RETRY_AFTER_INTERVAL))
-                    except ValueError:
-                        # retry-after could be formatted as absolute time
-                        # instead of seconds to wait. We don't know how to
-                        # parse that, but the apiserver doesn't generate
-                        # such responses anyway.
-                        seconds_to_wait = DEFAULT_RETRY_AFTER_INTERVAL
-                    if timeout:
-                        time_left = int(max(1, time_started + timeout - time.time()))
-                        seconds_to_wait = min(seconds_to_wait, time_left)
-                    logger.warn("%s %s: %s. Waiting %d seconds due to server unavailability..."
-                                % (method, url, exception_msg, seconds_to_wait))
+            exception_msg = _extract_msg_from_last_exception()
+            if isinstance(e, _expected_exceptions):
+                if response is not None and response.status == 503:
+                    seconds_to_wait = _extract_retry_after_timeout(response)
+                    logger.warn("%s %s: %s. Waiting %d seconds due to server unavailability...",
+                                method, url, exception_msg, seconds_to_wait)
                     time.sleep(seconds_to_wait)
                     # Note, we escape the "except" block here without
                     # incrementing try_index because 503 responses with
@@ -430,6 +476,7 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True, timeou
                 # Total number of allowed tries is the initial try + up to
                 # (max_retries) subsequent retries.
                 total_allowed_tries = max_retries + 1
+                ok_to_retry = False
                 # Because try_index is not incremented until we escape this
                 # iteration of the loop, try_index is equal to the number of
                 # tries that have failed so far, minus one. Test whether we
@@ -439,25 +486,26 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True, timeou
                        streaming_response_truncated:
                         ok_to_retry = always_retry or (method == 'GET') or _is_retryable_exception(e)
                     else:
-                        ok_to_retry = 500 <= response.status_code < 600
+                        ok_to_retry = 500 <= response.status < 600
 
-                    if ok_to_retry:
-                        if rewind_input_buffer_offset is not None:
-                            data.seek(rewind_input_buffer_offset)
-                        delay = 2 ** try_index
-                        logger.warn("%s %s: %s. Waiting %d seconds before retry %d of %d..."
-                                    % (method, url, exception_msg, delay, try_index + 1, max_retries))
-                        time.sleep(delay)
-                        try_index += 1
-                        continue
+                if ok_to_retry:
+                    if rewind_input_buffer_offset is not None:
+                        data.seek(rewind_input_buffer_offset)
+                    delay = min(2 ** try_index, DEFAULT_TIMEOUT)
+                    logger.warn("%s %s: %s. Waiting %d seconds before retry %d of %d...",
+                                method, url, exception_msg, delay, try_index + 1, max_retries)
+                    time.sleep(delay)
+                    try_index += 1
+                    continue
 
             # All retries have been exhausted OR the error is deemed not
-            # retryable. Propagate the latest error back to the caller.
+            # retryable. Print the latest error and propagate it back to the caller.
+            if not isinstance(e, exceptions.DXAPIError):
+                logger.error("%s %s: %s", method, url, exception_msg)
             raise
         finally:
             if success and try_index > 0:
-                logger.info("{} {}: Recovered after {} retries".format(method, url, try_index))
-
+                logger.info("%s %s: Recovered after %d retries", method, url, try_index)
 
         raise AssertionError('Should never reach this line: should have attempted a retry or reraised by now')
     raise AssertionError('Should never reach this line: should never break out of loop')
@@ -561,8 +609,6 @@ def set_project_context(dxid):
     global PROJECT_CONTEXT_ID
     PROJECT_CONTEXT_ID = dxid
 
-from .utils.env import get_env
-
 def get_auth_server_name(host_override=None, port_override=None):
     """
     Chooses the auth server name from the currently configured API server name.
@@ -578,50 +624,18 @@ def get_auth_server_name(host_override=None, port_override=None):
         return 'https://stagingauth.dnanexus.com'
     elif APISERVER_HOST == 'api.dnanexus.com':
         return 'https://auth.dnanexus.com'
+    elif APISERVER_HOST == "localhost" or APISERVER_HOST == "127.0.0.1":
+        if "DX_AUTHSERVER_HOST" not in os.environ or "DX_AUTHSERVER_PORT" not in os.environ:
+            err_msg = "Must set authserver env vars (DX_AUTHSERVER_HOST, DX_AUTHSERVER_PORT) if apiserver is {apiserver}."
+            raise exceptions.DXError(err_msg.format(apiserver=APISERVER_HOST))
+        else:
+            return os.environ["DX_AUTHSERVER_HOST"] + ":" + os.environ["DX_AUTHSERVER_PORT"]
     else:
         err_msg = "Could not determine which auth server is associated with {apiserver}."
         raise exceptions.DXError(err_msg.format(apiserver=APISERVER_HOST))
 
-def _initialize(suppress_warning=False):
-    '''
-    :param suppress_warning: Whether to suppress the warning message for any mismatch found in the environment variables and the dx configuration file
-    :type suppress_warning: boolean
-    '''
-    global _DEBUG, _UPGRADE_NOTIFY
-    try:
-        _DEBUG = int(os.environ.get('_DX_DEBUG', 0))
-    except ValueError as e:
-        print('WARNING: Expected _DX_DEBUG to be an integer, but got %r' % (os.environ['_DX_DEBUG'],),
-              file=sys.stderr)
-        _DEBUG = 0
-    _UPGRADE_NOTIFY = expanduser('~/.dnanexus_config/.upgrade_notify')
-    if os.path.exists(_UPGRADE_NOTIFY) and os.path.getmtime(_UPGRADE_NOTIFY) > time.time() - 86400: # 24 hours
-        _UPGRADE_NOTIFY = False
-
-    env_vars = get_env(suppress_warning)
-    for var in env_vars:
-        if env_vars[var] is not None:
-            os.environ[var] = env_vars[var]
-
-    set_api_server_info(host=os.environ.get("DX_APISERVER_HOST", None),
-                        port=os.environ.get("DX_APISERVER_PORT", None),
-                        protocol=os.environ.get("DX_APISERVER_PROTOCOL", None))
-
-    if "DX_SECURITY_CONTEXT" in os.environ:
-        set_security_context(json.loads(os.environ['DX_SECURITY_CONTEXT']))
-
-    if "DX_JOB_ID" in os.environ:
-        set_job_id(os.environ["DX_JOB_ID"])
-        if "DX_WORKSPACE_ID" in os.environ:
-            set_workspace_id(os.environ["DX_WORKSPACE_ID"])
-        if "DX_PROJECT_CONTEXT_ID" in os.environ:
-            set_project_context(os.environ["DX_PROJECT_CONTEXT_ID"])
-    else:
-        if "DX_PROJECT_CONTEXT_ID" in os.environ:
-            set_workspace_id(os.environ["DX_PROJECT_CONTEXT_ID"])
-            set_project_context(os.environ["DX_PROJECT_CONTEXT_ID"])
-
-_initialize()
+from .utils.config import DXConfig as _DXConfig
+config = _DXConfig()
 
 from .bindings import *
 from .dxlog import DXLogHandler
