@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -25,9 +25,9 @@ existing analysis ID.
 
 """
 
-from __future__ import (print_function, unicode_literals)
+from __future__ import print_function, unicode_literals, division, absolute_import
 
-import sys, time
+import time
 import dxpy
 from dxpy.bindings import (DXObject, )
 from dxpy.exceptions import DXJobFailureError
@@ -46,31 +46,14 @@ class DXAnalysis(DXObject):
     def __init__(self, dxid=None):
         self._test_harness_result = None
         DXObject.__init__(self, dxid=dxid)
+        self.set_id(dxid)
 
-    def set_id(self, dxid):
-        '''
-        :param dxid: Object ID
-        :type dxid: string
-
-        Discards the currently stored ID and associates the handler
-        with *dxid*.
-        '''
-        self._dxid = dxid
-
-    def get_id(self):
-        '''
-        :returns: Analysis ID of associated analysis
-        :rtype: string
-
-        Returns the analysis ID that the handler is currently
-        associated with.
-
-        '''
-
-        return self._dxid
-
-    def describe(self, **kwargs):
+    def describe(self, fields=None, **kwargs):
         """
+        :param fields: dict where the keys are field names that should
+            be returned, and values should be set to True (by default,
+            all fields are returned)
+        :type fields: dict
         :returns: Description of the analysis
         :rtype: dict
 
@@ -78,10 +61,12 @@ class DXAnalysis(DXObject):
         about the analysis
 
         """
-        self._desc = dxpy.api.analysis_describe(self._dxid, {}, **kwargs)
+        describe_input = {}
+        if fields is not None:
+            describe_input['fields'] = fields
+        self._desc = dxpy.api.analysis_describe(self._dxid, describe_input, **kwargs)
         return self._desc
 
-
     def add_tags(self, tags, **kwargs):
         """
         :param tags: Tags to add to the analysis
@@ -192,4 +177,4 @@ class DXAnalysis(DXObject):
 
         '''
 
-        return self.describe(**kwargs)["state"]
+        return self.describe(fields=dict(state=True), **kwargs)["state"]
