@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -45,11 +45,12 @@ becomes the sole developer of the app.
 
 """
 
-from __future__ import (print_function, unicode_literals)
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 import dxpy
 from . import DXObject, DXExecutable, DXJob, verify_string_dxid
 from ..exceptions import DXError
+from ..compat import basestring
 
 #########
 # DXApp #
