@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -21,22 +21,21 @@ DXFile Handler
 This remote file handler is a Python file-like object.
 '''
 
-from __future__ import (print_function, unicode_literals)
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 import os, sys, logging, traceback, hashlib, copy, time
 import concurrent.futures
+from multiprocessing import cpu_count
 
 import dxpy
 from . import DXDataObject
 from ..exceptions import DXFileError
 from ..utils import warn
+from ..utils.resolver import object_exists_in_project
 from ..compat import BytesIO
 
-if dxpy.snappy_available:
-    import snappy
 
-# TODO: adaptive buffer size
-DXFILE_HTTP_THREADS = 8
+DXFILE_HTTP_THREADS = min(cpu_count(), 8)
 DEFAULT_BUFFER_SIZE = 1024*1024*16
 if dxpy.JOB_ID:
     # Increase HTTP request buffer size when we are running within the
@@ -44,6 +43,7 @@ if dxpy.JOB_ID:
     DEFAULT_BUFFER_SIZE = 1024*1024*96
 
 MD5_READ_CHUNK_SIZE = 1024*1024*4
+FILE_REQUEST_TIMEOUT = 60
 
 class DXFile(DXDataObject):
     '''Remote file object handler.
@@ -88,6 +88,8 @@ class DXFile(DXDataObject):
     _http_threadpool = None
     _http_threadpool_size = DXFILE_HTTP_THREADS
 
+    NO_PROJECT_HINT = 'NO_PROJECT_HINT'
+
     @classmethod
     def set_http_threadpool_size(cls, num_threads):
         cls._http_threadpool_size = num_threads
@@ -391,7 +393,7 @@ class DXFile(DXDataObject):
         be in either the "open" or "closing" states.
         '''
 
-        return self.describe(**kwargs)["state"] == "closed"
+        return self.describe(fields={'state'}, **kwargs)["state"] == "closed"
 
     def close(self, block=False, **kwargs):
         '''
@@ -469,6 +471,9 @@ class DXFile(DXDataObject):
         else:
             md5.update(data)
 
+        req_input["md5"] = md5.hexdigest()
+        req_input["size"] = len(data)
+
         def get_upload_url_and_headers():
             # This function is called from within a retry loop, so to avoid amplifying the number of retries
             # geometrically, we decrease the allowed number of retries for the nested API call every time.
@@ -477,18 +482,24 @@ class DXFile(DXDataObject):
             elif kwargs['max_retries'] > 0:
                 kwargs['max_retries'] -= 1
 
+            if "timeout" not in kwargs:
+                kwargs["timeout"] = FILE_REQUEST_TIMEOUT
+
             resp = dxpy.api.file_upload(self._dxid, req_input, **kwargs)
             url = resp["url"]
-            headers = resp.get("headers", {})
-            headers['Content-Length'] = str(len(data))
-            headers['Content-MD5'] = md5.hexdigest()
-            return url, headers
+            return url, resp.get("headers", {})
 
         # The file upload API requires us to get a pre-authenticated upload URL (and headers for it) every time we
         # attempt an upload. Because DXHTTPRequest will retry requests under retryable conditions, we give it a callback
         # to ask us for a new upload URL every time it attempts a request (instead of giving them directly).
-        dxpy.DXHTTPRequest(get_upload_url_and_headers, data, jsonify_data=False, prepend_srv=False, always_retry=True,
-                           auth=None)
+        dxpy.DXHTTPRequest(get_upload_url_and_headers,
+                           data,
+                           jsonify_data=False,
+                           prepend_srv=False,
+                           always_retry=True,
+                           timeout=FILE_REQUEST_TIMEOUT,
+                           auth=None,
+                           method='PUT')
 
         self._num_uploaded_parts += 1
 
@@ -500,35 +511,57 @@ class DXFile(DXDataObject):
 
     def get_download_url(self, duration=24*3600, preauthenticated=False, filename=None, project=None, **kwargs):
         """
-        :param duration: number of seconds for which the generated URL will be valid
+        :param duration: number of seconds for which the generated URL will be
+            valid
         :type duration: int
-        :param preauthenticated: if True, generates a 'preauthenticated' download URL, which embeds authentication info in the URL and does not require additional headers
+        :param preauthenticated: if True, generates a 'preauthenticated'
+            download URL, which embeds authentication info in the URL and does
+            not require additional headers
         :type preauthenticated: bool
         :param filename: desired filename of the downloaded file
         :type filename: str
-        :param project: ID of a project containing the file (the download URL should be associated with this project)
+        :param project: ID of a project containing the file (the download URL
+            will be associated with this project, and this may affect which
+            billing account is billed for this download). If None, no hint is
+            supplied to the API server. If set to a project, the URL is only
+            valid so long as the user has access to that project and the
+            project contains that file.
         :type project: str
-        :returns: download URL and dict containing HTTP headers to be supplied with the request
+        :returns: download URL and dict containing HTTP headers to be supplied
+            with the request
         :rtype: tuple (str, dict)
+        :raises: :exc:`~dxpy.exceptions.ResourceNotFound` if the project does
+            not contain this file.
+
+        Obtains a URL that can be used to directly download the associated
+        file.
 
-        Obtains a URL that can be used to directly download the
-        associated file.
         """
+        # Test hook to write 'project' argument passed to API call to a
+        # local file
+        if '_DX_DUMP_BILLED_PROJECT' in os.environ:
+            with open(os.environ['_DX_DUMP_BILLED_PROJECT'], "w") as fd:
+                if project is not None:
+                    fd.write(project)
+
         args = {"duration": duration, "preauthenticated": preauthenticated}
         if filename is not None:
             args["filename"] = filename
         if project is not None:
             args["project"] = project
-        if self._download_url is None or self._download_url_expires > time.time():
+        if self._download_url is None or self._download_url_expires < time.time():
             # logging.debug("Download URL unset or expired, requesting a new one")
+            if "timeout" not in kwargs:
+                kwargs["timeout"] = FILE_REQUEST_TIMEOUT
             resp = dxpy.api.file_download(self._dxid, args, **kwargs)
             self._download_url = resp["url"]
             self._download_url_headers = resp.get("headers", {})
             self._download_url_expires = time.time() + duration - 60 # Try to account for drift
         return self._download_url, self._download_url_headers
 
-    def _generate_read_requests(self, start_pos=0, end_pos=None, **kwargs):
-        url, headers = self.get_download_url(**kwargs)
+    def _generate_read_requests(self, start_pos=0, end_pos=None, project=None, **kwargs):
+        # project=None means no hint is to be supplied to the apiserver. It is
+        # an error to supply a project that does not contain this file.
 
         if self._file_length == None:
             desc = self.describe(**kwargs)
@@ -552,6 +585,7 @@ class DXFile(DXDataObject):
                 i += 1
 
         for chunk_start_pos, chunk_end_pos in chunk_ranges(start_pos, end_pos):
+            url, headers = self.get_download_url(project=project, **kwargs)
             headers = copy.copy(headers)
             headers['Range'] = "bytes=" + str(chunk_start_pos) + "-" + str(chunk_end_pos)
             yield dxpy.DXHTTPRequest, [url, ''], {'method': 'GET',
@@ -560,6 +594,7 @@ class DXFile(DXDataObject):
                                                   'jsonify_data': False,
                                                   'prepend_srv': False,
                                                   'always_retry': True,
+                                                  'timeout': FILE_REQUEST_TIMEOUT,
                                                   'decode_response_body': False}
 
     def _next_response_content(self):
@@ -568,30 +603,43 @@ class DXFile(DXDataObject):
         if self._response_iterator is None:
             self._response_iterator = dxpy.utils.response_iterator(
                 self._request_iterator,
-                self._http_threadpool,
-                max_active_tasks=self._http_threadpool_size,
-                queue_id=id(self)
+                self._http_threadpool
             )
-        return next(self._response_iterator)
+        try:
+            return next(self._response_iterator)
+        except:
+            # If an exception is raised, the iterator is unusable for
+            # retrieving any more items. Destroy it so we'll reinitialize it
+            # next time.
+            self._response_iterator = None
+            self._request_iterator = None
+            raise
 
-    def read(self, length=None, use_compression=None, **kwargs):
+    def read(self, length=None, use_compression=None, project=None, **kwargs):
         '''
-        :param size: Maximum number of bytes to be read
-        :type size: integer
+        :param length: Maximum number of bytes to be read
+        :type length: integer
+        :param project: project to use as context for this download (may affect
+            which billing account is billed for this download). If specified,
+            must be a project in which this file exists. If not specified, the
+            project ID specified in the handler is used for the download, IF it
+            contains this file. If set to DXFile.NO_PROJECT_HINT, no project ID
+            is supplied for the download, even if the handler specifies a
+            project ID.
+        :type project: str or None
         :rtype: string
+        :raises: :exc:`~dxpy.exceptions.ResourceNotFound` if *project* is
+            supplied and it does not contain this file
 
-        Returns the next *size* bytes, or all the bytes until the end of
-        file (if no *size* is given or there are fewer than *size* bytes
-        left in the file).
+        Returns the next *length* bytes, or all the bytes until the end of file
+        (if no *length* is given or there are fewer than *length* bytes left in
+        the file).
 
-        .. note:: After the first call to read(), passthrough kwargs are
-           not respected while using the same response iterator (i.e.
-           until next seek).
+        .. note:: After the first call to read(), the project arg and
+           passthrough kwargs are not respected while using the same response
+           iterator (i.e. until next seek).
 
         '''
-        if self._response_iterator == None:
-            self._request_iterator = self._generate_read_requests(start_pos=self._pos, **kwargs)
-
         if self._file_length == None:
             desc = self.describe(**kwargs)
             if desc["state"] != "closed":
@@ -617,6 +665,26 @@ class DXFile(DXDataObject):
         if length == None or length > self._file_length - self._pos:
             length = self._file_length - self._pos
 
+        # Project specified explicitly to this method read(project=...) is
+        # treated strictly. If supplied, it must be a project in which this
+        # file exists. Otherwise, it's an error.
+        #
+        # If project=None, we fall back to the project attached to this handler
+        # (if any). If this is supplied, it's treated as a hint: if it's a
+        # project in which this file exists, it's passed on to the
+        # apiserver. Otherwise, NO hint is supplied. In principle supplying a
+        # project in the handler that doesn't contain this file ought to be an
+        # error, but it's this way for backwards compatibility. We don't know
+        # who might be doing downloads and creating handlers without being
+        # careful that the project encoded in the handler contains the file
+        # being downloaded. They may now rely on such behavior.
+        if project is None:
+            project_from_handler = self.get_proj_id()
+            if project_from_handler and object_exists_in_project(self.get_id(), project_from_handler):
+                project = project_from_handler
+        elif project == DXFile.NO_PROJECT_HINT:
+            project = None
+
         buf = self._read_buf
         buf_remaining_bytes = dxpy.utils.string_buffer_length(buf) - buf.tell()
         if length <= buf_remaining_bytes:
@@ -630,6 +698,10 @@ class DXFile(DXDataObject):
             while self._pos < orig_file_pos + length:
                 remaining_len = orig_file_pos + length - self._pos
 
+                if self._response_iterator is None:
+                    self._request_iterator = self._generate_read_requests(
+                        start_pos=self._pos, project=project, **kwargs)
+
                 if get_first_chunk_sequentially:
                     # Make the first chunk request without using the
                     # usual thread pool and block until it completes. On
@@ -637,8 +709,8 @@ class DXFile(DXDataObject):
                     # _next_response_content in the alternative block
                     # below. This starts the threadpool going for the
                     # second and all subsequent chunks.
-                    callable_, args, kwargs = next(self._request_iterator)
-                    content = callable_(*args, **kwargs)
+                    _callable, _args, _kwargs = next(self._request_iterator)
+                    content = _callable(*_args, **_kwargs)
                     get_first_chunk_sequentially = False
                 else:
                     content = self._next_response_content()
