@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -32,7 +32,7 @@ of :class:`~dxpy.bindings.dxproject.DXContainer`.
 
 """
 
-from __future__ import (print_function, unicode_literals)
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 import dxpy
 from . import DXObject
@@ -118,6 +118,11 @@ class DXContainer(DXObject):
         input hash to be supplied to each ``/describe`` call.
 
         """
+        # TODO: it would be nice if we could supply describe
+        # fields/defaultFields in a similar way to what we pass to the
+        # high-level describe method, rather than having to construct
+        # the literal API input
+
         api_method = dxpy.api.container_list_folder
         if isinstance(self, DXProject):
             api_method = dxpy.api.project_list_folder
@@ -174,12 +179,14 @@ class DXContainer(DXObject):
                                 "destination": destination},
                    **kwargs)
 
-    def remove_folder(self, folder, recurse=False, **kwargs):
+    def remove_folder(self, folder, recurse=False, force=False, **kwargs):
         """
         :param folder: Full path to the folder to remove
         :type folder: string
         :param recurse: If True, recursively remove all objects and subfolders in the folder
         :type recurse: bool
+        :param force: If True, will suppress errors for folders that do not exist
+        :type force: bool
 
         Removes the specified folder from the project or container. It
         must be empty to be removed, unless *recurse* is True.
@@ -194,13 +201,17 @@ class DXContainer(DXObject):
         if isinstance(self, DXProject):
             api_method = dxpy.api.project_remove_folder
 
-        api_method(self._dxid, {"folder": folder, "recurse": recurse},
+        api_method(self._dxid,
+                   {"folder": folder, "recurse": recurse, "force": force},
+                   always_retry=force,  # api call is idempotent under 'force' semantics
                    **kwargs)
 
-    def remove_objects(self, objects, **kwargs):
+    def remove_objects(self, objects, force=False, **kwargs):
         """
         :param objects: List of object IDs to remove from the project or container
         :type objects: list of strings
+        :param force: If True, will suppress errors for objects that do not exist
+        :type force: bool
 
         Removes the specified objects from the project or container.
 
@@ -213,7 +224,9 @@ class DXContainer(DXObject):
         if isinstance(self, DXProject):
             api_method = dxpy.api.project_remove_objects
 
-        api_method(self._dxid, {"objects": objects},
+        api_method(self._dxid,
+                   {"objects": objects, "force": force},
+                   always_retry=force,  # api call is idempotent under 'force' semantics
                    **kwargs)
 
     def clone(self, container, destination="/", objects=[], folders=[],
@@ -301,19 +314,22 @@ class DXProject(DXContainer):
             update_hash["version"] = version
         dxpy.api.project_update(self._dxid, update_hash, **kwargs)
 
-    def invite(self, invitee, level, **kwargs):
+    def invite(self, invitee, level, send_email=True, **kwargs):
         """
         :param invitee: Username (of the form "user-USERNAME") or email address of person to be invited to the project; use "PUBLIC" to make the project publicly available (in which case level must be set to "VIEW").
         :type invitee: string
         :param level: Permissions level that the invitee would get ("VIEW", "UPLOAD", "CONTRIBUTE", or "ADMINISTER")
         :type level: string
+        :param send_email: Determines whether user receives email notifications regarding the project invitation
+        :type send_email: boolean
 
         Invites the specified user to have access to the project.
 
         """
 
         return dxpy.api.project_invite(self._dxid,
-                                       {"invitee": invitee, "level": level},
+                                       {"invitee": invitee, "level": level,
+                                        "suppressEmailNotification": not send_email},
                                        **kwargs)
 
     def decrease_perms(self, member, level, **kwargs):
