@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -19,7 +19,7 @@ Records are the most basic data object and do not store additional data beyond t
 all data objects (details, properties, etc.).
 """
 
-from __future__ import (print_function, unicode_literals)
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 import dxpy
 from . import DXDataObject
@@ -76,12 +76,14 @@ class DXRecord(DXDataObject):
     _close = staticmethod(dxpy.api.record_close)
     _list_projects = staticmethod(dxpy.api.record_list_projects)
 
-    def _new(self, dx_hash, **kwargs):
+    def _new(self, dx_hash, close=False, **kwargs):
         """
         :param dx_hash: Standard hash populated in :func:`dxpy.bindings.DXDataObject.new()` containing attributes common to all data object classes.
         :type dx_hash: dict
         :param init_from: Record from which to initialize the metadata
         :type init_from: :class:`DXRecord`
+        :param close: Whether or not to close the record immediately after creating it
+        :type close: boolean
 
         Create a new remote record object.
 
@@ -95,5 +97,9 @@ class DXRecord(DXDataObject):
                     {"id": kwargs["init_from"].get_id(),
                      "project": kwargs["init_from"].get_proj_id()}
             del kwargs["init_from"]
+
+        if close:
+            dx_hash["close"] = True
+
         resp = dxpy.api.record_new(dx_hash, **kwargs)
         self.set_ids(resp["id"], dx_hash["project"])
