@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -24,12 +24,13 @@ be run together.  They can be run by calling the
 
 """
 
-from __future__ import (print_function, unicode_literals)
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 import re
 import dxpy
-from dxpy.bindings import DXDataObject, DXExecutable, DXAnalysis
-from dxpy.exceptions import DXError
+from ..bindings import DXDataObject, DXExecutable, DXAnalysis
+from ..exceptions import DXError
+from ..compat import basestring
 
 ##############
 # DXWorkflow #
