@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -22,15 +22,59 @@ provides search functionality over all data objects in the system. The
 (whether they are running, failed, or done).
 '''
 
-from __future__ import (print_function, unicode_literals)
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 import dxpy
 from . import DXApplet, DXApp, DXWorkflow, DXProject, DXJob, DXAnalysis
 from ..exceptions import DXError, DXSearchError
 
+
+def resolve_data_objects(objects, project=None, folder=None, batchsize=1000):
+    """
+    :param objects: Data object specifications, each with fields "name"
+                    (required), "folder", and "project"
+    :type objects: list of dictionaries
+    :param project: ID of project context; a data object's project defaults
+                    to this if not specified for that object
+    :type project: string
+    :param folder: Folder path within the project; a data object's folder
+                   path defaults to this if not specified for that object
+    :type folder: string
+    :param batchsize: Number of objects to resolve in each batch call to
+                      system_resolve_data_objects; defaults to 1000 and is
+                      only used for testing (must be a positive integer not
+                      exceeding 1000)
+    :type batchsize: int
+    :returns: List of results parallel to input objects, where each
+              entry is a list containing 0 or more dicts, each corresponding
+              to a resolved object
+    :rtype: List of lists of dictionaries
+
+    Each returned element is a list of dictionaries with keys "project" and
+    "id". The number of dictionaries for each element may be 0, 1, or more.
+    """
+    if not isinstance(batchsize, int) or batchsize <= 0 or batchsize > 1000:
+        raise ValueError("batchsize for resolve_data_objects must be a positive integer not exceeding 1000")
+    args = {}
+    if project:
+        args.update({'project': project})
+    if folder:
+        args.update({'folder': folder})
+
+    results = []
+
+    # Call API method /system/resolveDataObjects in groups of size batchsize
+    for i in range(0, len(objects), batchsize):
+        args.update({'objects': objects[i:(i+batchsize)]})
+        results.extend(dxpy.api.system_resolve_data_objects(args)['results'])
+    return results
+
+
 def _find(api_method, query, limit, return_handler, first_page_size, **kwargs):
-    ''' Takes an API method handler (dxpy.api.find...) and calls it with *query*, then wraps a generator around its
-    output. Used by the methods below.
+    ''' Takes an API method handler (dxpy.api.find*) and calls it with *query*,
+    and then wraps a generator around its output. Used by the methods below.
+
+    Note that this function may only be used for /system/find* methods.
     '''
     num_results = 0
 
@@ -69,11 +113,13 @@ def find_data_objects(classname=None, state=None, visibility=None,
                       link=None, project=None, folder=None, recurse=None,
                       modified_after=None, modified_before=None,
                       created_after=None, created_before=None,
-                      describe=None, limit=None, level=None,
+                      describe=False, limit=None, level=None,
                       return_handler=False, first_page_size=100,
                       **kwargs):
     """
-    :param classname: Class with which to restrict the search, i.e. one of "record", "file", "gtable", "table", "applet", "workflow"
+    :param classname:
+        Class with which to restrict the search, i.e. one of "record",
+        "file", "gtable", "applet", "workflow"
     :type classname: string
     :param state: State of the object ("open", "closing", "closed", "any")
     :type state: string
@@ -107,8 +153,12 @@ def find_data_objects(classname=None, state=None, visibility=None,
     :type created_after: int or string
     :param created_before: Timestamp before which each result was last created (see note below for interpretation)
     :type created_before: int or string
-    :param describe: Whether to also return the output of calling describe() on the object
-    :type describe: boolean
+    :param describe: Controls whether to also return the output of
+        calling describe() on each object. Supply False to omit describe
+        output, True to obtain the default describe output, or a dict to
+        be supplied as the describe call input (which may, among other
+        things, be used to customize the set of fields that is returned)
+    :type describe: bool or dict
     :param level: The minimum permissions level for which results should be returned (one of "VIEW", "UPLOAD", "CONTRIBUTE", or "ADMINISTER")
     :type level: string
     :param limit: The maximum number of results to be returned (if not specified, the number of results is unlimited)
@@ -129,9 +179,9 @@ def find_data_objects(classname=None, state=None, visibility=None,
        * A nonnegative integer, interpreted as milliseconds since the Epoch
        * A negative integer, interpreted as an offset in milliseconds relative
          to the current time
-       * A string with one of the suffixes "s", "m", "d", "w", or "y" (for
-         seconds, minutes, days, weeks, or years), interpreted as an offset
-         from the current time.
+       * A string containing a negative integer with one of the suffixes
+         "s", "m", "d", "w", or "y" (for seconds, minutes, days, weeks,
+         or years), interpreted as an offset from the current time.
 
        The following examples both find all items that were created more
        than 1 week ago::
@@ -202,7 +252,7 @@ def find_data_objects(classname=None, state=None, visibility=None,
             query["created"]["after"] = dxpy.utils.normalize_time_input(created_after)
         if created_before is not None:
             query["created"]["before"] = dxpy.utils.normalize_time_input(created_before)
-    if describe is not None:
+    if describe is not None and describe is not False:
         query["describe"] = describe
     if level is not None:
         query['level'] = level
@@ -211,6 +261,7 @@ def find_data_objects(classname=None, state=None, visibility=None,
 
     return _find(dxpy.api.system_find_data_objects, query, limit, return_handler, first_page_size, **kwargs)
 
+
 def find_executions(classname=None, launched_by=None, executable=None, project=None,
                     state=None, origin_job=None, parent_job=None, no_parent_job=False,
                     parent_analysis=None, no_parent_analysis=False, root_execution=None,
@@ -219,6 +270,10 @@ def find_executions(classname=None, launched_by=None, executable=None, project=N
                     first_page_size=100, return_handler=False, include_subjobs=True,
                     **kwargs):
     '''
+    :param classname:
+        Class with which to restrict the search, i.e. one of "job",
+        "analysis"
+    :type classname: string
     :param launched_by: User ID of the user who launched the execution's origin execution
     :type launched_by: string
     :param executable: ID of the applet or app that spawned this execution, or a corresponding remote object handler
@@ -243,8 +298,14 @@ def find_executions(classname=None, launched_by=None, executable=None, project=N
     :type created_after: int or string
     :param created_before: Timestamp before which each result was last created (see note accompanying :meth:`find_data_objects()` for interpretation)
     :type created_before: int or string
-    :param describe: Whether to also return the output of calling describe() on the execution. Besides supplying True (full description) or False (no details), you can also supply the dict {"io": False} to suppress detailed information about the execution's inputs and outputs.
-    :type describe: boolean or dict
+    :param describe: Controls whether to also return the output of
+        calling describe() on each execution. Supply False to omit
+        describe output, True to obtain the default describe output, or
+        a dict to be supplied as the describe call input (which may be
+        used to customize the set of fields that is to be returned; for
+        example, you can supply {"io": False} to suppress detailed
+        information about the execution's inputs and outputs)
+    :type describe: bool or dict
     :param name: Name of the job or analysis to search by (also see *name_mode*)
     :type name: string
     :param name_mode: Method by which to interpret the *name* field ("exact": exact match, "glob": use "*" and "?" as wildcards, "regexp": interpret as a regular expression)
@@ -330,7 +391,8 @@ def find_executions(classname=None, launched_by=None, executable=None, project=N
             query["created"]["after"] = dxpy.utils.normalize_time_input(created_after)
         if created_before is not None:
             query["created"]["before"] = dxpy.utils.normalize_time_input(created_before)
-    query["describe"] = describe
+    if describe is not None and describe is not False:
+        query["describe"] = describe
     if name is not None:
         if name_mode == 'exact':
             query['name'] = name
@@ -366,8 +428,9 @@ def find_analyses(*args, **kwargs):
     return find_executions(*args, **kwargs)
 
 def find_projects(name=None, name_mode='exact', properties=None, tags=None,
-                  level=None, describe=None, explicit_perms=None,
-                  public=None, billed_to=None, limit=None, return_handler=False, first_page_size=100, **kwargs):
+                  level=None, describe=False, explicit_perms=None,
+                  public=None, created_after=None, created_before=None, billed_to=None,
+                  limit=None, return_handler=False, first_page_size=100, **kwargs):
     """
     :param name: Name of the project (also see *name_mode*)
     :type name: string
@@ -379,12 +442,22 @@ def find_projects(name=None, name_mode='exact', properties=None, tags=None,
     :type tags: list of strings
     :param level: One of "VIEW", "UPLOAD", "CONTRIBUTE", or "ADMINSTER". If specified, only returns projects where the current user has at least the specified permission level.
     :type level: string
-    :param describe: Either false or the input to the describe call for the project
-    :type describe: boolean or dict
+    :param describe: Controls whether to also return the output of
+        calling describe() on each project. Supply False to omit
+        describe output, True to obtain the default describe output, or
+        a dict to be supplied as the describe call input (which may be
+        used to customize the set of fields that is returned)
+    :type describe: bool or dict
     :param explicit_perms: Filter on presence of an explicit permision. If True, matching projects must have an explicit permission (any permission granted directly to the user or an organization to which the user belongs). If False, matching projects must not have any explicit permissions for the user. (default is None, for no filter)
     :type explicit_perms: boolean or None
     :param public: Filter on the project being public. If True, matching projects must be public. If False, matching projects must not be public. (default is None, for no filter)
     :type public: boolean or None
+    :param created_after: Timestamp after which each result was created
+        (see note accompanying :meth:`find_data_objects()` for interpretation)
+    :type created_after: int or string
+    :param created_before: Timestamp before which each result was created
+        (see note accompanying :meth:`find_data_objects()` for interpretation)
+    :type created_before: int or string
     :param billed_to: Entity ID (user or organization) that pays for the project's storage costs
     :type billed_to: string
     :param limit: The maximum number of results to be returned (if not specified, the number of results is unlimited)
@@ -420,12 +493,18 @@ def find_projects(name=None, name_mode='exact', properties=None, tags=None,
         query["tags"] = {"$and": tags}
     if level is not None:
         query["level"] = level
-    if describe is not None:
+    if describe is not None and describe is not False:
         query["describe"] = describe
     if explicit_perms is not None:
         query['explicitPermission'] = explicit_perms
     if public is not None:
         query['public'] = public
+    if created_after is not None or created_before is not None:
+        query["created"] = {}
+        if created_after is not None:
+            query["created"]["after"] = dxpy.utils.normalize_time_input(created_after)
+        if created_before is not None:
+            query["created"]["before"] = dxpy.utils.normalize_time_input(created_before)
     if billed_to is not None:
         query['billTo'] = billed_to
     if limit is not None:
@@ -438,7 +517,7 @@ def find_apps(name=None, name_mode='exact', category=None,
               billed_to=None, created_by=None, developer=None,
               created_after=None, created_before=None,
               modified_after=None, modified_before=None,
-              describe=None, limit=None, return_handler=False, first_page_size=100, **kwargs):
+              describe=False, limit=None, return_handler=False, first_page_size=100, **kwargs):
     """
     :param name: Name of the app (also see *name_mode*)
     :type name: string
@@ -464,8 +543,12 @@ def find_apps(name=None, name_mode='exact', category=None,
     :type modified_after: int or string
     :param modified_before: Timestamp before which each result was last modified (see note accompanying :meth:`find_data_objects()` for interpretation)
     :type modified_before: int or string
-    :param describe: If True, also returns the output of calling describe() on the object
-    :type describe: boolean
+    :param describe: Controls whether to also return the output of
+        calling describe() on each app. Supply False to omit describe
+        output, True to obtain the default describe output, or a dict to
+        be supplied as the describe call input (which may be used to
+        customize the set of fields that is returned)
+    :type describe: bool or dict
     :param limit: The maximum number of results to be returned (if not specified, the number of results is unlimited)
     :type limit: int
     :param first_page_size: The number of results that the initial API call will return. Subsequent calls will raise this by multiplying by 2 up to a maximum of 1000.
@@ -515,7 +598,7 @@ def find_apps(name=None, name_mode='exact', category=None,
             query["created"]["after"] = dxpy.utils.normalize_time_input(created_after)
         if created_before is not None:
             query["created"]["before"] = dxpy.utils.normalize_time_input(created_before)
-    if describe is not None:
+    if describe is not None and describe is not False:
         query["describe"] = describe
     if limit is not None:
         query["limit"] = limit
@@ -540,44 +623,202 @@ def _find_one(method, zero_ok=False, more_ok=True, **kwargs):
             raise DXSearchError("Expected one result, but found none: "+str(kwargs))
     return result
 
+
 def find_one_data_object(zero_ok=False, more_ok=True, **kwargs):
     """
-    :param zero_ok: Specifies whether to raise an error or return None on 0 results for the search
-    :type zero_ok: boolean
-    :param more_ok: Specifies whether to raise an error on 2+ results for the search
-    :type more_ok: boolean
-    
-    Returns one data object that satisfies the supplied constraints. Supports all search constraint arguments supported
-    by :meth:`find_data_objects()`. If *zero_ok* is set to False (default), returns None if there are no results,
-    otherwise raises :class:`~dxpy.exceptions.DXSearchError`. If *more_ok* is set to False and more than one result is
-    returned for the search, also raises :class:`~dxpy.exceptions.DXSearchError`.
+    :param zero_ok:
+        If False (default), :class:`~dxpy.exceptions.DXSearchError` is
+        raised if the search has 0 results; if True, returns None if the
+        search has 0 results
+    :type zero_ok: bool
+    :param more_ok:
+        If False, :class:`~dxpy.exceptions.DXSearchError` is raised if
+        the search has 2 or more results
+    :type more_ok: bool
+
+    Returns one data object that satisfies the supplied constraints, or
+    None if none exist (provided *zero_ok* is True). Supports all search
+    constraint arguments supported by :meth:`find_data_objects()`.
+
     """
     return _find_one(find_data_objects, zero_ok=zero_ok, more_ok=more_ok, **kwargs)
 
+
 def find_one_project(zero_ok=False, more_ok=True, **kwargs):
     """
-    :param zero_ok: Specifies whether to raise an error or return None on 0 results for the search
-    :type zero_ok: boolean
-    :param more_ok: Specifies whether to raise an error on 2+ results for the search
-    :type more_ok: boolean
-    
-    Returns one project that satisfies the supplied constraints. Supports all search constraint arguments supported
-    by :meth:`find_projects()`. If *zero_ok* is set to False (default), returns None if there are no results,
-    otherwise raises :class:`~dxpy.exceptions.DXSearchError`. If *more_ok* is set to False and more than one result is
-    returned for the search, also raises :class:`~dxpy.exceptions.DXSearchError`.
+    :param zero_ok:
+        If False (default), :class:`~dxpy.exceptions.DXSearchError` is
+        raised if the search has 0 results; if True, returns None if the
+        search has 0 results
+    :type zero_ok: bool
+    :param more_ok:
+        If False, :class:`~dxpy.exceptions.DXSearchError` is raised if
+        the search has 2 or more results
+    :type more_ok: bool
+
+    Returns one project that satisfies the supplied constraints, or None
+    if none exist (provided *zero_ok* is True). Supports all search
+    constraint arguments supported by :meth:`find_projects()`.
+
     """
     return _find_one(find_projects, zero_ok=zero_ok, more_ok=more_ok, **kwargs)
 
+
 def find_one_app(zero_ok=False, more_ok=True, **kwargs):
     """
-    :param zero_ok: Specifies whether to raise an error or return None on 0 results for the search
-    :type zero_ok: boolean
-    :param more_ok: Specifies whether to raise an error on 2+ results for the search
-    :type more_ok: boolean
-    
-    Returns one app that satisfies the supplied constraints. Supports all search constraint arguments supported
-    by :meth:`find_apps()`. If *zero_ok* is set to False (default), returns None if there are no results,
-    otherwise raises :class:`~dxpy.exceptions.DXSearchError`. If *more_ok* is set to False and more than one result is
-    returned for the search, also raises :class:`~dxpy.exceptions.DXSearchError`.
+    :param zero_ok:
+        If False (default), :class:`~dxpy.exceptions.DXSearchError` is
+        raised if the search has 0 results; if True, returns None if the
+        search has 0 results
+    :type zero_ok: bool
+    :param more_ok:
+        If False, :class:`~dxpy.exceptions.DXSearchError` is raised if
+        the search has 2 or more results
+    :type more_ok: bool
+
+    Returns one app that satisfies the supplied constraints, or None if
+    none exist (provided *zero_ok* is True). Supports all search
+    constraint arguments supported by :meth:`find_apps()`.
+
     """
     return _find_one(find_apps, zero_ok=zero_ok, more_ok=more_ok, **kwargs)
+
+
+def _org_find(api_method, org_id, query, first_page_size=100):
+    """
+    Takes an API method handler ``dxpy.api.org_find...`` and calls it with
+    *org_id* and *query*, then wraps a generator around its output. Used by
+    :meth:`org_find_members` and :meth:`org_find_projects` below.
+
+    :param first_page_size: The number of results that the initial API call will return.
+    :type first_page_size: int
+
+    """
+    if "limit" not in query:
+        query["limit"] = min(first_page_size, 1000)
+
+    while True:
+        resp = api_method(org_id, query)
+        for result in resp["results"]:
+            yield result
+
+        # set up next query
+        if resp["next"] is not None:
+            query["starting"] = resp["next"]
+            query["limit"] = min(query["limit"] * 2, 1000)
+        else:
+            break
+
+
+def org_find_members(org_id=None, level=None, describe=False):
+    """
+    :param org_id: ID of the organization
+    :type org_id: string
+    :param level: The membership level in the org that each member in the result set must have (one of "MEMBER" or
+        "ADMIN")
+    :type level: string
+    :param describe: Whether or not to return the response of ``dxpy.api.user_describe`` for each result. False omits
+        the describe response; True includes it; a dict will be used as the input to ``dxpy.api.user_describe`` (to
+        customize the desired set of fields in the describe response).
+    :type describe: bool or dict
+
+    Returns a generator that yields all org members that match the query formed by intersecting all specified
+    constraints. The search is not restricted by any parameters that were unspecified.
+    """
+    query = {}
+    if level is not None:
+        query["level"] = level
+    query["describe"] = describe
+
+    return _org_find(dxpy.api.org_find_members, org_id, query)
+
+
+def org_find_projects(org_id=None, name=None, name_mode='exact', ids=None, properties=None, tags=None, describe=False,
+                      public=None, created_after=None, created_before=None):
+    """
+    :param org_id: ID of the organization
+    :type org_id: string
+    :param name: Name that each result must have (also see *name_mode* param)
+    :type name: string
+    :param name_mode: Method by which to interpret the *name* param ("exact": exact match,
+        "glob": use "*" and "?" as wildcards, "regexp": interpret as a regular expression)
+    :type name_mode: string
+    :param ids: List of project IDs. Each result must have a project ID that was specified in this list.
+    :type ids: array of strings
+    :param properties: Properties (key-value pairs) that each result must have
+        (use value True to require the property key and allow any value)
+    :type properties: dict
+    :param tags: Tags that each result must have
+    :type tags: list of strings
+    :param describe: Whether or not to return the response of ``dxpy.api.project_describe`` for each result. False
+        omits the describe response; True includes it; a dict will be used as the input to
+        ``dxpy.api.project_describe`` (to customize the desired set of fields in the describe response).
+    :type describe: bool or dict
+    :param public: True indicates that each result must be public; False indicates that each result must be private;
+        None indicates that both public and private projects will be returned in the result set.
+    :type public: boolean or None
+    :param created_after: Timestamp after which each result was created
+        (see note accompanying :meth:`find_data_objects()` for interpretation)
+    :type created_after: int or string
+    :param created_before: Timestamp before which each result was created
+        (see note accompanying :meth:`find_data_objects()` for interpretation)
+    :type created_before: int or string
+    :rtype: generator
+
+    Returns a generator that yields all projects that match the query formed by intersecting all specified
+    constraints. The search is not restricted by any parameters that were unspecified.
+
+    """
+    query = {}
+    if name is not None:
+        if name_mode == 'exact':
+            query['name'] = name
+        elif name_mode == 'glob':
+            query['name'] = {'glob': name}
+        elif name_mode == 'regexp':
+            query['name'] = {'regexp': name}
+        else:
+            raise DXError('org_find_projects: Unexpected value found for argument name_mode')
+    if ids is not None:
+        query["id"] = ids
+    if properties is not None:
+        if len(properties.keys()) == 1:
+            query["properties"] = properties
+        else:
+            query["properties"] = {"$and": [{k: v} for (k, v) in properties.iteritems()]}
+    if tags is not None:
+        if len(tags) == 1:
+            query["tags"] = tags[0]
+        else:
+            query["tags"] = {"$and": tags}
+    query["describe"] = describe
+    if public is not None:
+        query['public'] = public
+    if created_after is not None or created_before is not None:
+        query["created"] = {}
+        if created_after is not None:
+            query["created"]["after"] = dxpy.utils.normalize_time_input(created_after)
+        if created_before is not None:
+            query["created"]["before"] = dxpy.utils.normalize_time_input(created_before)
+
+    return _org_find(dxpy.api.org_find_projects, org_id, query)
+
+
+def find_orgs(query, first_page_size=10):
+    """
+    :param query: The input to the /system/findOrgs API method.
+    :type query: dict
+
+    :param first_page_size: The number of results that the initial
+        /system/findOrgs API call will return; default 10, max 1000. Subsequent
+        calls will raise the number of returned results exponentially up to a
+        max of 1000.
+    :type first_page_size: int
+
+    :rtype: generator
+
+    Returns a generator that yields all orgs matching the specified query. Will
+    transparently handle pagination as necessary.
+    """
+    return _find(dxpy.api.system_find_orgs, query, limit=None,
+                 return_handler=False, first_page_size=first_page_size)
