@@ -1,4 +1,4 @@
-# Copyright (C) 2014 DNAnexus, Inc.
+# Copyright (C) 2014-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -21,15 +21,13 @@ command-line client.
 'dx cp' works only between DIFFERENT projects. It will exit fatally otherwise.
 '''
 
-from __future__ import (print_function, unicode_literals)
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 import dxpy
 import requests
-import dxpy.utils.printing as printing
-from ..utils.resolver import (resolve_existing_path, resolve_path, is_analysis_id, is_hashid,
-                              get_last_pos_of_char, get_first_pos_of_char)
-from ..exceptions import (err_exit, DXCLIError, InvalidState)
-from . import (try_call, try_call_err_exit)
+from ..utils.resolver import (resolve_existing_path, resolve_path, is_hashid, get_last_pos_of_char)
+from ..exceptions import (err_exit, DXCLIError)
+from . import try_call
 from dxpy.utils.printing import (fill)
 
 
@@ -110,8 +108,7 @@ def cp_to_noexistent_destination(args, dest_path, dx_dest, dest_proj):
 
 
 def cp(args):
-    dest_proj, dest_path, _none = try_call(resolve_path,
-                                           args.destination, 'folder')
+    dest_proj, dest_path, _none = try_call(resolve_path, args.destination, expected='folder')
     if dest_path is None:
         raise DXCLIError('Cannot copy to a hash ID')
     dx_dest = dxpy.get_handler(dest_proj)
