@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -18,7 +18,7 @@
 Functions and classes used when launching platform executables from the CLI.
 '''
 
-from __future__ import (print_function, unicode_literals)
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 # TODO: refactor all dx run helper functions here
 
@@ -26,14 +26,13 @@ import os, sys, json, collections, pipes
 
 import dxpy
 from . import INTERACTIVE_CLI
-from ..exceptions import DXCLIError
-from ..utils.printing import (RED, GREEN, BLUE, YELLOW, WHITE, BOLD, ENDC, DELIMITER, UNDERLINE, get_delimiter, fill)
+from ..exceptions import DXCLIError, DXError
+from ..utils.printing import (RED, GREEN, WHITE, BOLD, ENDC, UNDERLINE, fill)
 from ..utils.describe import (get_find_executions_string, get_ls_l_desc, parse_typespec)
 from ..utils.resolver import (get_first_pos_of_char, is_hashid, is_job_id, is_localjob_id, paginate_and_pick, pick,
-                              resolve_existing_path, split_unescaped)
+                              resolve_existing_path, resolve_multiple_existing_paths, split_unescaped, is_analysis_id)
 from ..utils import OrderedDefaultdict
-from ..compat import input, str, shlex
-from ..utils.env import get_env_var
+from ..compat import input, str, shlex, basestring
 
 ####################
 # -i Input Parsing #
@@ -61,7 +60,7 @@ def parse_obj(string, klass):
         return {'$dnanexus_link': {"project": entity_result['describe']['project'],
                                    "id": entity_result['id']}}
 
-dx_data_classes = ['record', 'gtable', 'file', 'applet', 'table']
+dx_data_classes = ['record', 'gtable', 'file', 'applet', 'workflow']
 
 parse_input = {'boolean': parse_bool,
                'string': (lambda string: string),
@@ -72,9 +71,9 @@ parse_input = {'boolean': parse_bool,
                'gtable': (lambda string: parse_obj(string, 'gtable')),
                'file': (lambda string: parse_obj(string, 'file')),
                'applet': (lambda string: parse_obj(string, 'applet')),
+               'workflow': (lambda string: parse_obj(string, 'workflow')),
                'job': (lambda string: {'$dnanexus_link': string}),
-               'app': (lambda string: {'$dnanexus_link': string}),
-               'table': (lambda string: parse_obj(string, 'table'))}
+               'app': (lambda string: {'$dnanexus_link': string})}
 
 def _construct_jbor(job_id, field_name_and_maybe_index):
     '''
@@ -123,7 +122,7 @@ def interactive_help(in_class, param_desc, prompt):
             except:
                 pass
             if proj_name is not None:
-                print('Your current working directory is ' + proj_name + ':' + get_env_var('DX_CLI_WD', u'/'))
+                print('Your current working directory is ' + proj_name + ':' + dxpy.config.get('DX_CLI_WD', '/'))
         while True:
             print('Pick an option to find input data:')
             try:
@@ -415,6 +414,14 @@ class ExecutableInputs(object):
         self.required_inputs, self.optional_inputs, self.array_inputs = [], [], set()
         self.input_name_prefix = input_name_prefix
 
+        # List of tuples (input name, input value, input class, index), where input name and input value are
+        # propagated from command-line (input class is propagated from the input spec, and may be None if no input
+        # spec is provided). index is the order in which this particular input value is specified on the command-line
+        # relative to other input values of the same name (as multiple input values may be specified for the same
+        # input name). If input class is truthy, then the index is 0. Otherwise, if input name will have only a
+        # single input value instead of a list of input values, then index is -1.
+        self.requires_resolution = []
+
         if input_spec is None:
             input_spec = self._desc.get('inputSpec', [])
 
@@ -443,6 +450,66 @@ class ExecutableInputs(object):
         else:
             self.inputs.update(new_inputs)
 
+    def _update_requires_resolution_inputs(self):
+        input_paths = [quad[1] for quad in self.requires_resolution]
+        results = resolve_multiple_existing_paths(input_paths)
+        for input_name, input_value, input_class, input_index in self.requires_resolution:
+            project = results[input_value]['project']
+            folderpath = results[input_value]['folder']
+            entity_result = results[input_value]['name']
+            if input_class is None:
+                if entity_result is not None:
+                    if isinstance(entity_result, basestring):
+                        # Case: -ifoo=job-012301230123012301230123
+                        # Case: -ifoo=analysis-012301230123012301230123
+                        assert(is_job_id(entity_result) or
+                               (is_analysis_id(entity_result)))
+                        input_value = entity_result
+                    elif is_hashid(input_value):
+                        input_value = {'$dnanexus_link': entity_result['id']}
+                    elif 'describe' in entity_result:
+                        # Then findDataObjects was called (returned describe hash)
+                        input_value = {"$dnanexus_link": {"project": entity_result['describe']['project'],
+                                                          "id": entity_result['id']}}
+                    else:
+                        # Then resolveDataObjects was called in a batch (no describe hash)
+                        input_value = {"$dnanexus_link": {"project": entity_result['project'],
+                                                          "id": entity_result['id']}}
+                if input_index >= 0:
+                    if self.inputs[input_name][input_index] is not None:
+                        raise AssertionError("Expected 'self.inputs' to have saved a spot for 'input_value'.")
+                    self.inputs[input_name][input_index] = input_value
+                else:
+                    if self.inputs[input_name] is not None:
+                        raise AssertionError("Expected 'self.inputs' to have saved a spot for 'input_value'.")
+                    self.inputs[input_name] = input_value
+            else:
+                msg = 'Value provided for input field "' + input_name + '" could not be parsed as ' + \
+                      input_class + ': '
+                if input_value == '':
+                    raise DXCLIError(msg + 'empty string cannot be resolved')
+                if entity_result is None:
+                    raise DXCLIError(msg + 'could not resolve \"' + input_value + '\" to a name or ID')
+                try:
+                    dxpy.bindings.verify_string_dxid(entity_result['id'], input_class)
+                except DXError as details:
+                    raise DXCLIError(msg + str(details))
+                if is_hashid(input_value):
+                    input_value = {'$dnanexus_link': entity_result['id']}
+                elif 'describe' in entity_result:
+                    # Then findDataObjects was called (returned describe hash)
+                    input_value = {'$dnanexus_link': {"project": entity_result['describe']['project'],
+                                                      "id": entity_result['id']}}
+                else:
+                    # Then resolveDataObjects was called in a batch (no describe hash)
+                    input_value = {"$dnanexus_link": {"project": entity_result['project'],
+                                                      "id": entity_result['id']}}
+                if input_index != -1:
+                    # The class is an array, so append the resolved value
+                    self.inputs[input_name].append(input_value)
+                else:
+                    self.inputs[input_name] = input_value
+
     def add(self, input_name, input_value):
         if self.input_name_prefix is not None:
             if input_name.startswith(self.input_name_prefix):
@@ -463,53 +530,92 @@ class ExecutableInputs(object):
                 input_class = self.input_spec[input_name]['class']
 
         if input_class is None:
-            done = False
+            resolved_input_as_jbor = False
             try:
                 # Resolve "job-xxxx:output-name" syntax into a canonical job ref
                 job_id, field = split_unescaped(':', input_value)
-                if is_job_id(job_id) or is_localjob_id(job_id):
-                    input_value = _construct_jbor(job_id, field)
-                    done = True
             except:
                 pass
-            if not done:
+            else:
+                if is_job_id(job_id) or is_localjob_id(job_id):
+                    input_value = _construct_jbor(job_id, field)
+                    resolved_input_as_jbor = True
+
+            if resolved_input_as_jbor:
+                if isinstance(self.inputs[input_name], list):
+                    self.inputs[input_name].append(input_value)
+                else:
+                    self.inputs[input_name] = input_value
+            else:
                 try:
                     parsed_input_value = json.loads(input_value, object_pairs_hook=collections.OrderedDict)
-                    if type(parsed_input_value) in (collections.OrderedDict, list, int, long, float):
-                        input_value = parsed_input_value
-                    else:
+                    if type(parsed_input_value) not in (collections.OrderedDict, list, int, long, float):
                         raise Exception()
                 except:
                     # Not recognized JSON (list or dict), so resolve it as a name
-                    try:
-                        project, folderpath, entity_result = resolve_existing_path(input_value,
-                                                                                   expected='entity')
-                    except:
-                        # If not possible, then leave it as a string
-                        project, folderpath, entity_result = None, None, None
-                    if entity_result is not None:
-                        if is_hashid(input_value):
-                            input_value = {'$dnanexus_link': entity_result['id']}
-                        else:
-                            input_value = {"$dnanexus_link": {"project": entity_result['describe']['project'],
-                                                              "id": entity_result['id']}}
-            if isinstance(self.inputs[input_name], list) and \
-               not isinstance(self.inputs[input_name], basestring):
-                self.inputs[input_name].append(input_value)
-            else:
-                self.inputs[input_name] = input_value
+                    # Add to self.requires_resolution, and insert None as a placeholder in self.inputs;
+                    # self.requires_resolution will also store the location of the corresponding placeholder
+                    if isinstance(self.inputs[input_name], list):
+                        self.requires_resolution.append((input_name, input_value, None, len(self.inputs[input_name])))
+                        self.inputs[input_name].append(None)
+                    else:
+                        # If the input is to only have a single value, then the index will be -1
+                        self.requires_resolution.append((input_name, input_value, None, -1))
+                        self.inputs[input_name] = None
+                else:
+                    if isinstance(self.inputs[input_name], list):
+                        self.inputs[input_name].append(parsed_input_value)
+                    else:
+                        self.inputs[input_name] = parsed_input_value
         else:
             # Input class is known.  Respect the "array" class.
-
+            val_substrings = split_unescaped(':', input_value)
             try:
-                input_value = parse_input_or_jbor(input_class, input_value)
-            except Exception as details:
-                raise DXCLIError('Value provided for input field "' + input_name + '" could not be parsed as ' + input_class + ': ' + str(details))
-
-            if input_class.startswith('array:'):
-                self.inputs[input_name].append(input_value)
-            else:
-                self.inputs[input_name] = input_value
+                if len(val_substrings) == 2 and (is_job_id(val_substrings[0]) or is_localjob_id(val_substrings[0])):
+                    input_value = _construct_jbor(val_substrings[0], val_substrings[1])
+                    if input_class.startswith('array:'):
+                        self.inputs[input_name].append(input_value)
+                    else:
+                        self.inputs[input_name] = input_value
+                else:
+                    # TODO: Consolidate the following checks with exec_io.parse_input_or_jbor
+                    # `parse_bool()` can throw DXCLIError, which will be
+                    # propagated directly to the caller.
+                    if input_class == 'boolean':
+                        self.inputs[input_name] = parse_bool(input_value)
+                    elif input_class == 'array:boolean':
+                        self.inputs[input_name].append(parse_bool(input_value))
+                    elif input_class == 'string':
+                        self.inputs[input_name] = input_value
+                    elif input_class == 'array:string':
+                        self.inputs[input_name].append(input_value)
+                    elif input_class == 'float':
+                        self.inputs[input_name] = float(input_value)
+                    elif input_class == 'array:float':
+                        self.inputs[input_name].append(float(input_value))
+                    elif input_class == 'int':
+                        self.inputs[input_name] = int(input_value)
+                    elif input_class == 'array:int':
+                        self.inputs[input_name].append(int(input_value))
+                    elif input_class == 'hash':
+                        self.inputs[input_name] = json.loads(input_value)
+                    elif input_class == 'array:hash':
+                        self.inputs[input_name].append(json.loads(input_value))
+                    elif input_class == 'job' or input_class == 'app':
+                        self.inputs[input_name] = {'$dnanexus_link': input_value}
+                    elif input_class == 'array:job' or input_class == 'array:app':
+                        self.inputs[input_name].append({'$dnanexus_link': input_value})
+                    else:
+                        # Add to self.requires_resolution
+                        if input_class.startswith('array:'):
+                            # No placeholders needed; just pass in input_index that is not -1 to append a result
+                            self.requires_resolution.append((input_name, input_value, input_class[6:], 0))
+                        else:
+                            # input_name is to only have a single value, set index to -1
+                            self.requires_resolution.append((input_name, input_value, input_class, -1))
+            except (ValueError, TypeError) as details:
+                raise DXCLIError('Value provided for input field "' + input_name + '" could not be parsed as ' +
+                                 input_class + ': ' + str(details))
 
     def init_completer(self):
         try:
@@ -622,6 +728,7 @@ class ExecutableInputs(object):
                     raise DXCLIError('An input was found that did not conform to the syntax: -i<input name>=<input value>')
                 self.add(self.executable._get_input_name(name) if \
                          self._desc.get('class') == 'workflow' else name, value)
+            self._update_requires_resolution_inputs()
 
         if self.input_spec is None:
             for i in self.inputs:
