@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -24,6 +24,7 @@ import string
 import random
 import sys
 import argparse
+import os
 
 # to find the magic library
 import magic
@@ -581,10 +582,14 @@ def import_BED(**args):
     current_file = 1
 
     for import_filename in split_on_track(bed_filename_uncomp):
+        try:
+            bed_basename = os.path.basename(bed_filename)
+        except:
+            bed_basename = bed_filename
         if current_file == 1:
-            name = bed_filename
+            name = bed_basename
         else:
-            name = bed_filename+"_"+str(current_file)
+            name = bed_basename+"_"+str(current_file)
         current_file += 1
         bed_type = detect_type(import_filename)["type"]
         delimiter = detect_type(import_filename)["delimiter"]
