@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -16,7 +16,7 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-from __future__ import print_function, unicode_literals
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 import logging
 logging.basicConfig(level=logging.WARNING)
@@ -30,15 +30,16 @@ import tempfile
 import time
 from datetime import datetime
 import dxpy, dxpy.app_builder
-from dxpy import logger
+from .. import logger
 
-from dxpy.utils import json_load_raise_on_duplicates
-from dxpy.utils.resolver import resolve_path, is_container_id
-from dxpy.utils.completer import LocalCompleter
-from dxpy.app_categories import APP_CATEGORIES
-from dxpy.exceptions import err_exit, DXError
-from dxpy.utils.printing import BOLD
-from dxpy.compat import open, USING_PYTHON2, decode_command_line_args
+from ..utils import json_load_raise_on_duplicates
+from ..utils.resolver import resolve_path, is_container_id
+from ..utils.completer import LocalCompleter
+from ..app_categories import APP_CATEGORIES
+from ..cli import try_call
+from ..exceptions import err_exit, DXError
+from ..utils.printing import BOLD
+from ..compat import open, USING_PYTHON2, decode_command_line_args, basestring
 
 decode_command_line_args()
 
@@ -50,6 +51,12 @@ app_options = parser.add_argument_group('options for creating apps', '(Only vali
 applet_options = parser.add_argument_group('options for creating applets', '(Only valid when --app/--create-app is NOT specified)')
 
 # COMMON OPTIONS
+parser.add_argument("--ensure-upload", help="If specified, will bypass computing checksum of " +
+                                            "resources directory and upload it unconditionally; " +
+                                            "by default, will compute checksum and upload only if " +
+                                            "it differs from a previously uploaded resources bundle.",
+                    action="store_true")
+
 src_dir_action = parser.add_argument("src_dir", help="App or applet source directory (default: current directory)", nargs='?')
 src_dir_action.completer = LocalCompleter()
 
@@ -137,6 +144,10 @@ parser.add_argument("--no-json", help=argparse.SUPPRESS, action="store_false", d
 parser.add_argument("--extra-args", help="Arguments (in JSON format) to pass to the /applet/new API method, overriding all other settings")
 parser.add_argument("--run", help="Run the app or applet after building it (options following this are passed to "+BOLD("dx run")+"; run at high priority by default)", nargs=argparse.REMAINDER)
 
+# --region
+app_options.add_argument("--region", help="The region this app will be created in")
+
+
 class DXSyntaxError(Exception):
     def __init__(self, message):
         self.message = message
@@ -189,7 +200,7 @@ def parse_destination(dest_str):
     # [PROJECT]:/FOLDER/
     # [PROJECT]:/ENTITYNAME
     # [PROJECT]:/FOLDER/ENTITYNAME
-    return resolve_path(dest_str)
+    return try_call(resolve_path, dest_str)
 
 
 def _lint(dxapp_json_filename, mode):
@@ -483,7 +494,7 @@ def _parse_app_spec(src_dir):
 def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
                       version_override=None, bill_to_override=None, dx_toolkit_autodep="stable",
                       do_version_autonumbering=True, do_try_update=True, do_parallel_build=True,
-                      do_check_syntax=True):
+                      do_check_syntax=True, region=None):
     if mode == 'app':
         builder_app = 'app-tarball_app_builder'
     else:
@@ -559,7 +570,14 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
 
     elif mode == "app":
         using_temp_project_for_remote_build = True
-        build_project_id = dxpy.api.project_new({"name": "dx-build-app --remote temporary project"})["id"]
+        try:
+            if region:
+                build_project_id = dxpy.api.project_new({"name": "dx-build-app --remote temporary project",
+                                                         "region": region})["id"]
+            else:
+                build_project_id = dxpy.api.project_new({"name": "dx-build-app --remote temporary project"})["id"]
+        except:
+            err_exit()
 
     try:
         # Resolve relative paths and symlinks here so we have something
@@ -592,7 +610,7 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
                     if sys.stderr.isatty():
                         if last_console_update > 0:
                             sys.stderr.write("\r")
-                        sys.stderr.write("Compressing target directory {dir}... ({kb_so_far:,} kb)".format(dir=src_dir, kb_so_far=total_num_bytes / 1024))
+                        sys.stderr.write("Compressing target directory {dir}... ({kb_so_far:,} kb)".format(dir=src_dir, kb_so_far=total_num_bytes // 1024))
                         sys.stderr.flush()
                         last_console_update = current_time
                     elif not printed_static_message:
@@ -663,7 +681,9 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
                              version_override=None, bill_to_override=None, use_temp_build_project=True,
                              do_parallel_build=True, do_version_autonumbering=True, do_try_update=True,
                              dx_toolkit_autodep="stable", do_check_syntax=True, dry_run=False,
-                             return_object_dump=False, confirm=True, **kwargs):
+                             return_object_dump=False, confirm=True, ensure_upload=False, region=None, **kwargs):
+
+    dxpy.app_builder.build(src_dir, parallel_build=do_parallel_build)
     app_json = _parse_app_spec(src_dir)
 
     _verify_app_source_dir(src_dir, mode, enforce=do_check_syntax)
@@ -679,7 +699,14 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
         working_project, override_folder, override_applet_name = parse_destination(destination_override)
     elif mode == "app" and use_temp_build_project and not dry_run:
         # Create a temp project
-        working_project = dxpy.api.project_new({"name": "Temporary build project for dx-build-app"})["id"]
+        try:
+            if region:
+                working_project = dxpy.api.project_new({"name": "Temporary build project for dx-build-app",
+                                                        "region": region})["id"]
+            else:
+                working_project = dxpy.api.project_new({"name": "Temporary build project for dx-build-app"})["id"]
+        except:
+            err_exit()
         logger.debug("Created temporary project %s to build in" % (working_project,))
         using_temp_project = True
 
@@ -707,14 +734,14 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
             for result in dxpy.find_data_objects(classname="applet", name=dest_name, folder=dest_folder,
                                                  project=dest_project, recurse=False):
                 dest_path = dest_folder + dest_name
-                raise dxpy.app_builder.AppBuilderException("An applet already exists at {} (id {}) and the " +
-                                                           "--overwrite (-f) or --archive (-a) options were not " +
-                                                           "given".format(dest_path, result['id']))
+                msg = "An applet already exists at {} (id {}) and neither".format(dest_path, result["id"])
+                msg += " -f/--overwrite nor -a/--archive were given."
+                raise dxpy.app_builder.AppBuilderException(msg)
 
-        dxpy.app_builder.build(src_dir, parallel_build=do_parallel_build)
-
-        bundled_resources = dxpy.app_builder.upload_resources(
-            src_dir, project=working_project, folder=override_folder) if not dry_run else []
+        bundled_resources = dxpy.app_builder.upload_resources(src_dir,
+                                                              project=working_project,
+                                                              folder=override_folder,
+                                                              ensure_upload=ensure_upload) if not dry_run else []
 
         try:
             # TODO: the "auto" setting is vestigial and should be removed.
@@ -817,9 +844,11 @@ def _build_app(args, extra_args):
                 do_try_update=args.update,
                 dx_toolkit_autodep=args.dx_toolkit_autodep,
                 do_check_syntax=args.check_syntax,
+                ensure_upload=args.ensure_upload,
                 dry_run=args.dry_run,
                 confirm=args.confirm,
                 return_object_dump=args.json,
+                region=args.region,
                 **extra_args
                 )
 
@@ -882,7 +911,9 @@ def _build_app(args, extra_args):
         if not args.check_syntax:
             more_kwargs['do_check_syntax'] = False
 
-        return _build_app_remote(args.mode, args.src_dir, destination_override=args.destination, publish=args.publish, dx_toolkit_autodep=args.dx_toolkit_autodep, **more_kwargs)
+        return _build_app_remote(args.mode, args.src_dir, destination_override=args.destination,
+                                 publish=args.publish, dx_toolkit_autodep=args.dx_toolkit_autodep,
+                                 region=args.region, **more_kwargs)
 
 
 def main(**kwargs):
@@ -918,6 +949,9 @@ def main(**kwargs):
     if args.mode == "app" and args.destination != '.':
         parser.error("--destination cannot be used when creating an app (only an applet)")
 
+    if args.mode == "applet" and args.region:
+        parser.error("--region cannot be used when creating an applet (only an app)")
+
     if args.dx_toolkit_autodep in ['beta', 'unstable']:
         logging.warn('The --dx-toolkit-beta-autodep and --dx-toolkit-unstable-autodep flags have no effect and will be removed at some date in the future.')
 
