@@ -1,4 +1,4 @@
-# Copyright (C) 2014 DNAnexus, Inc.
+# Copyright (C) 2014-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -23,7 +23,7 @@ and generating an equivalent applet source directory.
 
 '''
 
-from __future__ import print_function, unicode_literals
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 import collections
 import json
