@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -18,7 +18,7 @@
 Utilities used in the DNAnexus execution environment and test harness.
 """
 
-from __future__ import print_function, unicode_literals
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 import os, sys, json, re, collections, logging, argparse, string, itertools, subprocess, tempfile
 from functools import wraps
@@ -300,7 +300,7 @@ class DXExecDependencyInstaller(object):
             print(message)
 
     def generate_shellcode(self, dep_group):
-        base_apt_shellcode = "apt-get install --yes --no-install-recommends {p}"
+        base_apt_shellcode = "export DEBIAN_FRONTEND=noninteractive && apt-get install --yes --no-install-recommends {p}"
         dx_apt_update_shellcode = "apt-get update -o Dir::Etc::sourcelist=sources.list.d/nucleus.list -o Dir::Etc::sourceparts=- -o APT::Get::List-Cleanup=0"
         change_apt_archive = r"sed -i -e s?http://.*.ec2.archive.ubuntu.com?http://us.archive.ubuntu.com? /etc/apt/sources.list"
         apt_err_msg = "APT failed, retrying with full update against ubuntu.com"
@@ -328,8 +328,21 @@ class DXExecDependencyInstaller(object):
         elif dep_type == "cpan":
             return "cpanm --notest " + make_pm_atoms(packages, version_separator="~")
         elif dep_type == "cran":
-            r_shellcode = "R -e 'die <- function() { q(status=1) }; options(error=die); options(warn=2); install.packages(commandArgs(trailingOnly=TRUE), repos=\"http://cran.us.r-project.org\")' --args "
-            return r_shellcode + make_pm_atoms(packages)
+            repo = "http://cran.us.r-project.org"
+            r_preamble = "die <- function() { q(status=1) }; options(error=die); options(warn=2);"
+            r_preamble += 'r <- getOption("repos"); r["CRAN"] = "{repo}"; options(repos=r)'.format(repo=repo)
+            r_cmd_template = "R -e '{preamble}; {cmd}'"
+            bootstrap_cmd = 'install.packages("devtools")'
+            commands = []
+            for package in packages:
+                if "version" in package:
+                    cmd = 'require(devtools); install_version("{name}", version="{version}")'.format(**package)
+                else:
+                    cmd = 'install.packages("{}")'.format(package["name"])
+                commands.append(r_cmd_template.format(preamble=r_preamble, cmd=cmd))
+            if any("require(devtools)" in cmd for cmd in commands):
+                commands.insert(0, r_cmd_template.format(preamble=r_preamble, cmd=bootstrap_cmd))
+            return " && ".join(commands)
         elif dep_type == "git":
             commands = ["apt-get install --yes git make", "export GIT_SSH=dx-git-ssh-helper"]
             for dep in packages:
@@ -386,9 +399,6 @@ class DXExecDependencyInstaller(object):
     def _validate_dependency(self, dep):
         if "name" not in dep:
             raise DXExecDependencyError('Expected field "name" to be present in execution dependency "{}"'.format(dep))
-        if dep.get("package_manager") == "cran" and "version" in dep:
-            msg = 'Execution dependency {} has a "version" field, but versioning is not supported for CRAN dependencies'
-            raise DXExecDependencyError(msg.format(dep))
         elif dep.get("package_manager") == "git" and "url" not in dep:
             raise DXExecDependencyError('Execution dependency "{}" does not have a "url" field'.format(dep))
 
