@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -19,8 +19,8 @@ This submodule gives basic utilities for printing to the terminal.
 '''
 
 import textwrap, subprocess, os, sys
-from .env import sys_encoding
-from ..compat import USING_PYTHON2
+import json
+from ..compat import USING_PYTHON2, sys_encoding
 from ..exceptions import DXCLIError
 
 if sys.stdout.isatty():
@@ -156,3 +156,34 @@ def refill_paragraphs(string, ignored_prefix='    '):
     paragraphs = string.split('\n\n')
     refilled_paragraphs = [fill(paragraph) if not paragraph.startswith(ignored_prefix) else paragraph for paragraph in paragraphs]
     return '\n\n'.join(refilled_paragraphs).strip('\n')
+
+
+def _format_find_projects_results(results):
+    for result in results:
+        print(result["id"] + DELIMITER(" : ") + result['describe']['name'] +
+              DELIMITER(' (') + result["level"] + DELIMITER(')'))
+
+
+def _format_find_org_members_results(results):
+    for result in results:
+        print(result["id"] + DELIMITER(" : ") + result['describe']['first'] + DELIMITER(' ') +
+              result['describe']['last'] + DELIMITER(' ') + DELIMITER(' (') + result["level"] +
+              DELIMITER(')'))
+
+
+def format_find_results(args, results):
+    """
+    Formats the output of ``dx find ...`` commands for `--json` and `--brief` arguments; also formats if no formatting
+    arguments are given.
+    Currently used for ``dx find projects``, ``dx find org_projects``, and ``dx find org_members``
+    """
+    if args.json:
+        print(json.dumps(list(results), indent=4))
+    elif args.brief:
+        for result in results:
+            print(result['id'])
+    else:
+        if args.func.__name__ in ("find_projects", "org_find_projects"):
+            _format_find_projects_results(results)
+        if args.func.__name__ in ("org_find_members"):
+            _format_find_org_members_results(results)
