@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -17,17 +17,25 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-from __future__ import print_function, unicode_literals
+from __future__ import print_function, unicode_literals, division, absolute_import
 
 import os, sys, unittest, json, tempfile, subprocess
 import pexpect
+import pipes
 
 from dxpy_testutil import DXTestCase, check_output
 import dxpy_testutil as testutil
 
 import dxpy
 from dxpy.scripts import dx_build_app
-from dxpy.exceptions import DXCLIError
+
+
+def run(command, **kwargs):
+    print("$ %s" % (command,))
+    output = check_output(command, shell=True, **kwargs)
+    print(output)
+    return output
+
 
 supported_languages = ['Python', 'C++', 'bash']
 
@@ -360,6 +368,47 @@ class TestDXAppWizardAndRunAppLocally(DXTestCase):
                 subprocess.check_output(['dx', 'build', appdir, '--destination', applet_name])
                 subprocess.check_output(['dx', 'run', applet_name, '-y', '--wait'] + cmdline_args)
 
+    @unittest.skipUnless(testutil.TEST_ENV, 'skipping test that would clobber your local environment')
+    def test_dx_run_app_locally_without_auth(self):
+        temp_file_path = tempfile.mkdtemp()
+        app_spec = {
+            "name": "test_run_locally_without_auth",
+            "dxapi": "1.0.0",
+            "runSpec": {"file": "code.py", "interpreter": "python2.7"},
+            "inputSpec": [{"name": "foo", "class": "file"}],
+            "outputSpec": [],
+            "version": "1.0.0"
+            }
+        app_dir_path = os.path.join(temp_file_path, app_spec['name'])
+        os.mkdir(app_dir_path)
+        with open(os.path.join(app_dir_path, 'dxapp.json'), 'w') as manifest:
+            manifest.write(json.dumps(app_spec))
+        with open(os.path.join(app_dir_path, 'code.py'), 'w') as code_file:
+            code_file.write('')
+        with testutil.without_auth(), testutil.without_project_context():
+            with self.assertSubprocessFailure(stderr_regexp="logged in", exit_code=3):
+                run("dx-run-app-locally " + pipes.quote(app_dir_path) + " -ifoo=nothing")
+
+    def test_dx_run_app_locally_invalid_interpreter(self):
+        temp_file_path = tempfile.mkdtemp()
+        app_spec = {
+            "name": "test_run_locally_invalid_interpreter",
+            "dxapi": "1.0.0",
+            "runSpec": {"file": "code.py", "interpreter": "python"},
+            "inputSpec": [],
+            "outputSpec": [],
+            "version": "1.0.0"
+            }
+        app_dir_path = os.path.join(temp_file_path, app_spec['name'])
+        os.mkdir(app_dir_path)
+        with open(os.path.join(app_dir_path, 'dxapp.json'), 'w') as manifest:
+            manifest.write(json.dumps(app_spec))
+        with open(os.path.join(app_dir_path, 'code.py'), 'w') as code_file:
+            code_file.write('')
+        with self.assertSubprocessFailure(stderr_regexp="Unknown interpreter python", exit_code=3):
+            run("dx-run-app-locally " + pipes.quote(app_dir_path))
+
+
 '''
 test the upload/download helpers by running them locally
 '''
@@ -449,5 +498,9 @@ class TestDXBashHelpers(DXTestCase):
         dxpy.upload_string("ABCD", wait_on_close=True, name="B.txt")
         self.run_test_app_locally('parseq', ["-iseq1=A.txt", "-iseq2=B.txt", "-iref=A.txt", "-iref=B.txt"])
 
+    def test_file_optional(self):
+        ''' Tests that file optional input arguments are handled correctly '''
+        self.run_test_app_locally('file_optional', ["-icreate_seq3=true"])
+
 if __name__ == '__main__':
     unittest.main()
