@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -32,10 +32,10 @@ class TestDXFS(unittest.TestCase):
             return
         proj_name = u"dxclient_test_pröject"
         cls.project_id = subprocess.check_output(u"dx new project '{p}' --brief".format(p=proj_name), shell=True).strip()
-        os.environ["DX_PROJECT_CONTEXT_ID"] = cls.project_id
-        os.environ["DX_CLI_WD"] = '/'
+        dxpy.config["DX_PROJECT_CONTEXT_ID"] = cls.project_id
+        dxpy.config["DX_CLI_WD"] = '/'
         cls.project = dxpy.DXProject(cls.project_id)
-        dxpy._initialize(suppress_warning=True)
+        dxpy.config.__init__(suppress_warning=True)
 
         subprocess.check_call(['dx', 'mkdir', 'foo'])
         subprocess.check_call(['dx', 'mkdir', 'bar'])
@@ -87,15 +87,15 @@ class TestDXFS(unittest.TestCase):
         os.rmdir(os.path.join(self.mountpoint, 'xyz'))
         self.assertNotIn('/xyz', self.project.list_folder('/')['folders'])
 
-    def test_dxfs_write(self):
-        filename1 = os.path.join(self.mountpoint, 'foo', 'f1')
-        with open(filename1, 'w') as fh:
-            fh.write('0123456789ABCDEF'*256)
-        subprocess.check_call(['dxfs', 'close', filename1, '--wait'])
-        with open(filename1) as fh:
-            d = fh.read()
-            print len(d)
-            self.assertEqual(d, '0123456789ABCDEF'*256, "File readback failed")
+    #def test_dxfs_write(self):
+    #    filename1 = os.path.join(self.mountpoint, 'foo', 'f1')
+    #    with open(filename1, 'w') as fh:
+    #        fh.write('0123456789ABCDEF'*256)
+    #    subprocess.check_call(['dxfs', 'close', filename1, '--wait'])
+    #    with open(filename1) as fh:
+    #        d = fh.read()
+    #        print len(d)
+    #        self.assertEqual(d, '0123456789ABCDEF'*256, "File readback failed")
 
 if __name__ == '__main__':
     unittest.main()
