@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013-2014 DNAnexus, Inc.
+# Copyright (C) 2013-2015 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -17,12 +17,14 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-from __future__ import print_function, unicode_literals
+from __future__ import print_function, unicode_literals, division, absolute_import
 
-import unittest, time, json, re
+import unittest, time, json, re, os
+import dateutil.parser
+import dxpy
 from dxpy import AppError, AppInternalError, DXFile, DXRecord
 from dxpy.utils import (describe, exec_utils, genomic_utils, response_iterator, get_futures_threadpool, DXJSONEncoder,
-                        normalize_timedelta)
+                        normalize_timedelta, normalize_time_input, config)
 from dxpy.utils.exec_utils import DXExecDependencyInstaller
 from dxpy.compat import USING_PYTHON2
 
@@ -51,16 +53,16 @@ class TestErrorSanitizing(unittest.TestCase):
         # ASCII str
         self.assertEqual(exec_utils._safe_unicode(ValueError("foo")), "foo")
         # UTF-8 encoded str
-        self.assertEqual(exec_utils._safe_unicode(ValueError(u"crème".encode("utf-8"))),
-                         u"cr\xe8me" if USING_PYTHON2 else u"b'cr\\xc3\\xa8me'")
+        self.assertEqual(exec_utils._safe_unicode(ValueError("crème".encode("utf-8"))),
+                         "cr\xe8me" if USING_PYTHON2 else "b'cr\\xc3\\xa8me'")
         # Unicode obj
-        self.assertEqual(exec_utils._safe_unicode(ValueError(u"brûlée")), u"br\xfbl\xe9e")
+        self.assertEqual(exec_utils._safe_unicode(ValueError("brûlée")), "br\xfbl\xe9e")
         # Not UTF-8
         if USING_PYTHON2:
             expected = "Invalid read name: D??n?x?s [Raw error message: 496e76616c69642072656164206e616d653a2044d1c16ee878fb73]"
         else:
             expected = "b'Invalid read name: D\\xd1\\xc1n\\xe8x\\xfbs'"
-        self.assertEqual(exec_utils._safe_unicode(ValueError(u"Invalid read name: DÑÁnèxûs".encode("ISO-8859-1"))), expected)
+        self.assertEqual(exec_utils._safe_unicode(ValueError("Invalid read name: DÑÁnèxûs".encode("ISO-8859-1"))), expected)
 
     def test_formatting_exceptions(self):
         self.assertEqual(exec_utils._format_exception_message(ValueError("foo")), "ValueError: foo")
@@ -69,11 +71,11 @@ class TestErrorSanitizing(unittest.TestCase):
 class TestGenomicUtils(unittest.TestCase):
     def test_reverse_complement(self):
         self.assertEqual(b"TTTTAAACCG", genomic_utils.reverse_complement(b"CGGTTTAAAA"))
-        self.assertEqual(b"TTTTAAACCG", genomic_utils.reverse_complement(u"CGGTTTAAAA"))
+        self.assertEqual(b"TTTTAAACCG", genomic_utils.reverse_complement("CGGTTTAAAA"))
         self.assertEqual(b"TTTTAAACCG", genomic_utils.reverse_complement(b"cggtttaaaa"))
-        self.assertEqual(b"TTTTAAACCG", genomic_utils.reverse_complement(u"cggtttaaaa"))
+        self.assertEqual(b"TTTTAAACCG", genomic_utils.reverse_complement("cggtttaaaa"))
         self.assertEqual(b"NNNNNTTTTAAACCG", genomic_utils.reverse_complement(b"CGGTTTAAAANNNNN"))
-        self.assertEqual(b"NNNNNTTTTAAACCG", genomic_utils.reverse_complement(u"CGGTTTAAAANNNNN"))
+        self.assertEqual(b"NNNNNTTTTAAACCG", genomic_utils.reverse_complement("CGGTTTAAAANNNNN"))
         with self.assertRaises(ValueError):
             genomic_utils.reverse_complement("oops")
 
@@ -97,13 +99,6 @@ class TestResponseIterator(unittest.TestCase):
         for i, res in enumerate(response_iterator(tasks(), get_futures_threadpool(5), max_active_tasks=6)):
             self.assertEqual(i, res)
 
-        def tasks2():
-            for i in range(8):
-                yield task, [i], {"sleep_for": (8-i)/8.0}
-
-        for i, res in enumerate(response_iterator(tasks2(), get_futures_threadpool(5), num_retries=2, retry_after=0.1)):
-            self.assertEqual(i, res)
-
 class TestDXUtils(unittest.TestCase):
     def test_dxjsonencoder(self):
         f = DXFile("file-" + "x"*24, project="project-" + "y"*24)
@@ -141,8 +136,9 @@ class TestDXExecDependsUtils(unittest.TestCase):
         with self.assertRaisesRegexp(AppInternalError, 'Expected field "name" to be present'):
             get_edi({"dependencies": [{"foo": "bar"}]})
 
-        with self.assertRaisesRegexp(AppInternalError, "versioning is not supported for CRAN dependencies"):
-            get_edi({"dependencies": [{"name": "foo", "package_manager": "cran", "version": "1.2.3"}]})
+        edi = get_edi({"dependencies": [{"name": "foo", "package_manager": "cran", "version": "1.2.3"}]})
+        edi.install()
+        assert_cmd_ran(edi, "R -e .+ install.packages.+devtools.+install_version.+foo.+version.+1.2.3")
 
         with self.assertRaisesRegexp(AppInternalError, 'does not have a "url" field'):
             get_edi({"dependencies": [{"name": "foo", "package_manager": "git"}]})
@@ -165,7 +161,9 @@ class TestDXExecDependsUtils(unittest.TestCase):
                                         {"name": "Module::Provision", "package_manager": "cpan", "version": "0.36.1"},
                                         {"name": "LWP::MediaTypes", "package_manager": "cpan"},
                                         {"name": "RJSONIO", "package_manager": "cran"},
-                                        {"name": "ggplot2", "package_manager": "cran", "stages": ["main"]},
+                                        {"name": "plyr", "package_manager": "cran", "version": "1.8.1"},
+                                        {"name": "ggplot2", "package_manager": "cran", "stages": ["main"],
+                                         "version": "1.0.1"},
                                         {"name": "r1", "id": {"$dnanexus_link": "record-123"}},
                                         {"name": "g1",
                                          "package_manager": "git",
@@ -181,7 +179,7 @@ class TestDXExecDependsUtils(unittest.TestCase):
         assert_cmd_ran(edi, re.escape("pip install --upgrade pytz==2014.7 certifi"))
         assert_cmd_ran(edi, "apt-get install --yes --no-install-recommends tmux")
         assert_cmd_ran(edi, re.escape("gem install rake --version 10.3.2 && gem install nokogiri"))
-        assert_cmd_ran(edi, "R -e .+ install.packages.+ --args RJSONIO ggplot2")
+        assert_cmd_ran(edi, "R -e .+ install.packages.+\"RJSONIO\".+install_version.+\"ggplot2\".+version=\"1.0.1\"")
         assert_log_contains(edi, 'Skipping bundled dependency "r1" because it does not refer to a file')
         assert_cmd_ran(edi, re.escape("cd $(mktemp -d) && git clone https://github.com/dnanexus/oauth2-demo"))
         assert_cmd_ran(edi, "cd /tmp/ee-edi-test-bwa && git clone https://github.com/dnanexus/bwa")
@@ -200,6 +198,7 @@ class TestDXExecDependsUtils(unittest.TestCase):
         edi.install()
         assert_cmd_ran(edi, "apt-get install --yes --no-install-recommends git")
 
+
 class TestTimeUtils(unittest.TestCase):
     def test_normalize_timedelta(self):
         for i, o in (("-15", -15000),
@@ -212,5 +211,79 @@ class TestTimeUtils(unittest.TestCase):
                      ("-1w", -1000*60*60*24*7)):
             self.assertEqual(normalize_timedelta(i), o)
 
+    def test_strings_vs_ints(self):
+        # This is potentially confusing; it would be nice to have two
+        # separate methods, or a flag to control the interpretation of
+        # the field when a string with no suffix is supplied.
+
+        # "dx login" can supply this form
+        self.assertEqual(normalize_time_input("1414141414", default_unit='s'), 1414141414000)   # interpreted as sec
+        # "dx find ... --created-*" can supply this form
+        self.assertEqual(normalize_time_input("1234567890"), 1234567890)  # interpreted as ms
+        self.assertEqual(normalize_time_input(1414141414000), 1414141414000)  # interpreted as ms
+
+    def test_normalize_time_input(self):
+        # TODO: Add tests for negative time inputs e.g. "-12345", -12345, "-5d"
+
+        for i, o in ((12345678, 12345678),
+                     ("0", 0),
+                     ("12345678", 12345678),
+                     ("15s", 15 * 1000),
+                     ("1d", (24 * 60 * 60 * 1000)),
+                     ("0w", 0),
+                     ("2015-10-01", int(time.mktime(dateutil.parser.parse("2015-10-01").timetuple()) * 1000))):
+            self.assertEqual(normalize_time_input(i), o)
+
+        # Test default_unit='s'
+        for i, o in ((12345678, 12345678 * 1000),
+                     ("12345678", 12345678 * 1000),
+                     ("15s", 15 * 1000),
+                     ("1d", (24 * 60 * 60 * 1000)),
+                     ("0w", 0),
+                     ("2015-10-01", int(time.mktime(dateutil.parser.parse("2015-10-01").timetuple()) * 1000))):
+            self.assertEqual(normalize_time_input(i, default_unit='s'), o)
+
+        with self.assertRaises(ValueError):
+            normalize_time_input("1223*")
+        with self.assertRaises(ValueError):
+            normalize_time_input("12345", default_unit='h')
+        with self.assertRaises(ValueError):
+            normalize_time_input(12345, default_unit='h')
+        with self.assertRaises(ValueError):
+            normalize_time_input("1234.5678")
+        with self.assertRaises(ValueError):
+            normalize_time_input(1234.5678)
+
+
+class TestDXConfig(unittest.TestCase):
+    def test_dxconfig(self):
+        environ_backup = os.environ.copy()
+        try:
+            c = config.DXConfig()
+            for var in c.VAR_NAMES:
+                value = '{"foo": "bar"}'
+                c[var] = value
+                if var in c.CORE_VAR_NAMES - {"DX_SECURITY_CONTEXT", "DX_WORKSPACE_ID"}:
+                    self.assertEqual(getattr(dxpy, var.lstrip("DX_")), value)
+                elif var == "DX_SECURITY_CONTEXT":
+                    self.assertEqual(json.dumps(dxpy.SECURITY_CONTEXT), value)
+                c.write(var, None)
+                del c[var]
+            c.update(DX_CLI_WD="/wd")
+            self.assertIn("DX_CLI_WD", c)
+            self.assertEqual(c["DX_CLI_WD"], "/wd")
+            self.assertEqual(c.pop("DX_CLI_WD"), "/wd")
+            self.assertEqual(c.pop("DX_CLI_WD", None), None)
+            self.assertIn("DXConfig object at", repr(c))
+            c.update(c.defaults)
+            self.assertEqual(len(c), len(list(c)))
+            del c[c.defaults.keys()[0]]
+            self.assertEqual(len(c), len(list(c)))
+            dxpy.config["DX_PROJECT_CONTEXT_NAME"] = None
+            self.assertEqual(os.environ["DX_PROJECT_CONTEXT_NAME"], "")
+        finally:
+            os.environ.update(environ_backup)
+            dxpy.config.__init__(suppress_warning=True)
+
 if __name__ == '__main__':
     unittest.main()
